<?php
/**
 * Plugin Name: Enquir3 Feedback
 * Plugin URI: https://www.enquir3.com/wordpress-feedback-plugin
 * Description: Display Enquir3 Feedback Information.
 * Author: Enquir3 Ltd
 * Author URI: https://www.enquir3.com
 * Version: 1.0.7
 */

if (!defined('EQ3F_PLUGIN_PATH')) {
  define( 'EQ3F_PLUGIN_PATH', dirname( __FILE__ ) . '/' );
}
define( 'EQ3F_PLUGIN_BASENAME', basename( __FILE__ ) );

define( 'EQ3F_TAXONOMY_TERM_DELETE_FAILURE', 1001 );
define( 'EQ3F_TAXONOMY_TERM_DELETE_SUCCESS', 1002 );

require_once EQ3F_PLUGIN_PATH . 'includes/feedback-settings-new.inc';

$eq3f_options_new = get_option( 'enquir3_feedback_options_new' );
if ( ! $eq3f_options_new ) {
	eq3f_set_default_options_new();
	$eq3f_options_new = get_option( 'enquir3_feedback_options_new' );
}

$eq3f_feedback = new EQ3F_Feedback( $eq3f_options_new );

/**
 * Class: EQ3F_Feedback
 * Run the following to check coding standards:
 * php ~/bin/phpcs.phar --standard=WordPress enquir3-feedback.php
 */
class EQ3F_Feedback {

	/**
	 * Instance of feedback client.
	 *
	 * @var RemstoFeedbackClient $feedback_client
	 */
	protected $feedback_client;

	/**
	 * Client ID.
	 *
	 * @var int $site_id
	 */
	protected $site_id;

	/**
	 * Client configuration.
	 *
	 * @var Array $conf
	 */
	protected $conf;

	/**
	 * Constructor
	 *
	 * @param type $conf Configration.
	 */
	public function __construct( $conf ) {

		$this->conf    = $conf;
		$this->site_id = $conf['client_id'];
		if ( '' !== $this->site_id ) {
			add_shortcode( 'enquir3-feedback-test', array( $this, 'enquir3_feedback_test' ) );
			add_shortcode( 'enquir3-feedback-midcopy', array( $this, 'enquir3_feedback_midcopy' ) );
			add_shortcode( 'enquir3-feedback-tabs', array( $this, 'enquir3_feedback_tabs' ) );
			add_shortcode( 'enquir3-feedback-listing-tabs', array( $this, 'enquir3_feedback_listing_tabs' ) );
			add_shortcode( 'enquir3-feedback-listing-performance', array( $this, 'enquir3_feedback_listing_performance' ) );
			add_shortcode( 'enquir3-feedback-listing-summary', array( $this, 'enquir3_feedback_listing_summary' ) );
			add_shortcode( 'enquir3-feedback-listing-rating', array( $this, 'enquir3_feedback_listing_rating' ) );
			add_shortcode( 'enquir3-feedback-listing-total-stars', array( $this, 'enquir3_feedback_listing_total_stars' ) );
			add_shortcode( 'enquir3-feedback-listing-number-ratings', array( $this, 'enquir3_feedback_listing_number_ratings' ) );
			add_shortcode( 'enquir3-feedback-form', array( $this, 'enquir3_feedback_entry_form_new' ) );
			add_shortcode( 'enquir3-feedback-form-new', array( $this, 'enquir3_feedback_entry_form_new' ) );
			add_action( 'wp_enqueue_scripts', array( $this, 'remsto_feedback_js' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'remsto_feedback_js' ) );
			add_action( 'wp_enqueue_scripts', array( $this, 'remsto_feedback_css' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'remsto_feedback_css' ) );
			add_action( 'widgets_init', array( $this, 'remsto_feedback_create_widgets' ) );
			add_action( 'wp_ajax_remsto_feedback_ajax', array( $this, 'remsto_feedback_ajax' ) );
			add_action( 'wp_ajax_nopriv_remsto_feedback_ajax', array( $this, 'remsto_feedback_ajax' ) );
			add_action( 'admin_post_taxonomy_form_submit', array( $this, 'taxonomy_form_submit' ) );
			add_action( 'admin_post_taxonomy_term_form_submit', array( $this, 'taxonomy_term_form_submit' ) );
			add_action( 'admin_post_taxonomy_term_delete_form_submit', array( $this, 'taxonomy_term_delete_form_submit' ) );
			add_action( 'admin_post_feedback_delete_form_submit', array( $this, 'feedback_delete_form_submit' ) );
			add_filter( 'gglcptch_add_custom_form', array( $this, 'add_custom_recaptcha_forms' ) );
			add_filter( 'the_title', array( $this, 'feedback_page_title' ), 10, 2 );
			add_action( 'parse_request', array( $this, 'remsto_feedback_pages' ) );
    }

    add_action( 'init', array( $this, 'remsto_feedback_routes' ) );
		add_action( 'admin_init', array( $this, 'add_admin_init' ) );
		add_action( 'admin_menu', array( $this, 'add_menu_items' ) );
		add_action( 'plugins_loaded', array( $this, 'init' ) );
		add_action( 'phpmailer_init', array( $this, 'phpmailer_setfrom' ) );
	}

	/**
	 * Activate
	 */
	public static function activate() {
		if ( ! is_plugin_active( 'google-captcha/google-captcha.php' ) ) {
			// Deactivate the plugin.
			deactivate_plugins( basename( __FILE__ ) );
			// Throw an error in the WordPress admin console.
			$error_message = __( 'This plugin requires the "reCaptcha by BestWebSoft" plugin to be active!' );
			die( esc_attr( $error_message ) );
		}

		// Create thank you page.
		$id                          = self::create_thankyou_page();
		$options                     = eq3f_get_options_new();
		$options['thankyou_page_id'] = $id;
		update_option( 'enquir3_feedback_options_new', $options );

		// Create other required pages.
		self::create_tabs_page();
		self::create_form_page();

		// Add capabilities.
		self::add_capability();
	}

	/**
	 * Deactivate
	 */
	public static function deactivate() {
		// Remove capabilities.
		self::remove_capability();
	}

	/**
	 * Initialise plug-in
	 */
	public function init() {
		require_once EQ3F_PLUGIN_PATH . 'includes/class-eq3f-feedback-management-table.php';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClient.inc';
		if ( ! class_exists( 'RemstoFeedbackClient' ) ) {
			require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoFeedbackClient.inc';
		}
		if ( ! class_exists( 'RemstoListingClient' ) ) {
			require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoListingClient.inc';
		}
		//if ( ! class_exists( 'RemstoListingClient' ) ) {
			//require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoListingClient.inc';
		//}
		require_once EQ3F_PLUGIN_PATH . 'includes/class-eq3f-feedback-taxonomy-list.php';
		require_once EQ3F_PLUGIN_PATH . 'includes/class-eq3f-feedback-taxonomy-term-list.php';
		require_once EQ3F_PLUGIN_PATH . 'includes/class-eq3f-feedback-taxonomy-edit-form.php';
		require_once EQ3F_PLUGIN_PATH . 'includes/class-eq3f-feedback-taxonomy-term-edit-form.php';
		require_once EQ3F_PLUGIN_PATH . 'includes/feedback-midcopy.inc';
		require_once EQ3F_PLUGIN_PATH . 'widgets/class-eq3f-widget-performance.php';
		require_once EQ3F_PLUGIN_PATH . 'widgets/class-eq3f-widget-rating.php';
		require_once EQ3F_PLUGIN_PATH . 'widgets/class-eq3f-widget-summary.php';
		require_once EQ3F_PLUGIN_PATH . 'widgets/class-eq3f-latest-feedback-widget.php';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClientTaxonomyDisplayOptions.inc';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClientTaxonomyTypes.inc';

		$this->feedback_client = new RemstoFeedbackClient( $this->conf );

		$this->display_messages();
	}

	/**
	 * Register JavaScript
	 */
	public function remsto_feedback_js() {
		wp_register_script(
			'eq3f-more-popup-script',
			plugins_url( '/js/more_popup.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-feedback-ajax-client-script',
			plugins_url( '/js/remsto_ajax_client.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-feedback_form',
			plugins_url( '/js/feedback_form.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-toggle-content',
			plugins_url( '/js/toggle_content.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-video-dialog',
			plugins_url( '/js/video_dialog.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-jquery-jcarousel',
			plugins_url( '/js/jquery.jcarousel.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-feedback-scroller',
			plugins_url( '/js/feedback_scroller.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-feedback-ajax-buttons',
			plugins_url( '/js/ajax_buttons.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-tabs-script',
			plugins_url( '/js/remsto_tabs.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-forms',
			plugins_url( 'js/remsto_forms.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
		wp_register_script(
			'eq3f-collapse-settings-section',
			plugins_url( 'js/collapse_settings_section.js', __FILE__ ),
			array( 'jquery' ),
			'1.0',
			true
		);
	}

	/**
	 * Register CSS
	 */
	public function remsto_feedback_css() {
		wp_register_style(
			'eq3f-base-style',
			plugins_url( '/css/remsto-base.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-item-style',
			plugins_url( '/css/feedback-item.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-icon-style',
			plugins_url( '/css/remsto_feedback_icons.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-feedback',
			plugins_url( '/css/remsto_feedback.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-performance-widget',
			plugins_url( '/css/performance-widget.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-rating-widget',
			plugins_url( '/css/remsto-rating.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-feeback-recent-widget',
			plugins_url( '/css/remsto-feeback-recent-widget.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-feedback-overview',
			plugins_url( '/css/remsto_feedback_overview.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-feedback-midcopy',
			plugins_url( '/css/remsto_mid_copy.css', __FILE__ ),
			array(),
			'1.0'
		);
		wp_register_style(
			'eq3f-feedback-admin',
			plugins_url( '/css/remsto_feedback_admin.css', __FILE__ ),
			array(),
			'1.0'
		);
	}

	/**
	 * Get data via Ajax
	 */
	public function remsto_feedback_ajax() {
		require_once EQ3F_PLUGIN_PATH . '/includes/class-eq3f-feedback-ajax.php';
		$feedback_ajax = new EQ3F_Feedback_Ajax( $this->feedback_client );
		$output        = $feedback_ajax->get_data();
		$allowed_html  = array(
			'div'  => array(
				'id'      => array(),
				'class'   => array(),
				'style'   => array(),
				'title'   => array(),
				'data-id' => array(),

			),
			'span' => array(
				'id'    => array(),
				'class' => array(),
				'title' => array(),
			),
			'a'    => array(
				'href'   => array(),
				'target' => array(),
			),
		);
		echo wp_kses( $output, $allowed_html );
		wp_die();
	}

	/**
	 * Get feedback client
	 */
	public function get_feedback_client() {
		return $this->feedback_client;
	}

	/**
	 * Run some tests
	 *
	 * @param array $atts attributes.
	 *
	 * @return string
	 */
	public function enquir3_feedback_test( $atts ) {
		$id = self::create_thankyou_page();
		if ( is_int( $id ) && $id > 0 ) {
			$options                     = eq3f_get_options_new();
			$options['thankyou_page_id'] = $id;
			update_option( 'enquir3_feedback_options_new', $options );
		}

		return '';
	}

	/**
	 * Thank you page
	 *
	 * @return string
	 */
	private static function create_thankyou_page() {
		$existing_thankyou_page = get_page_by_title( 'Thank You' );
		if ( $existing_thankyou_page ) {
			if ( 'trash' !== $existing_thankyou_page->post_status ) {
				return $existing_thankyou_page->ID;
			}
		}

		$thankyou_page = array(
			'post_type'    => 'page',
			'post_title'   => 'Thank You',
			'post_content' => '<div>Thank you for your feedback!</div><a href="/feedback">View feedback</a>',
			'post_status'  => 'publish',
			'post_slug'    => 'thankyou',
		);

		return wp_insert_post( $thankyou_page );
	}

	/**
	 * Feedback Tabs Page
	 *
	 * @return int ID of tabs page
	 */
	private static function create_tabs_page() {

		$existing_tabs_page = get_page_by_title( 'Feedback' );

		if ( $existing_tabs_page ) {
			if ( 'trash' !== $existing_tabs_page->post_status ) {
				return $existing_tabs_page->ID;
			}
		}

		$new_page = array(
			'post_type'    => 'page',
			'post_title'   => 'Feedback',
			'post_content' => '[enquir3-feedback-tabs]',
			'post_status'  => 'publish',
			'post_slug'    => 'feedback',
		);

		return wp_insert_post( $new_page );
	}

	/**
	 * Create feedback form page
	 *
	 * @return int ID of form page
	 */
	private static function create_form_page() {

		$existing_form_page = get_page_by_title( 'Feedback Form' );

		if ( $existing_form_page ) {
			if ( 'trash' !== $existing_form_page->post_status ) {
				return $existing_form_page->ID;
			}
		}

		$new_page = array(
			'post_type'    => 'page',
			'post_title'   => 'Feedback Form',
			'post_content' => '[enquir3-feedback-form]',
			'post_status'  => 'publish',
			'post_slug'    => 'feedback-form',
		);

		return wp_insert_post( $new_page );
	}

	/**
	 * Mid copy short code process
	 *
	 * @param array $atts Unused.
	 * @return string Mid copy HTML
	 */
	public function enquir3_feedback_midcopy( $atts ) {

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );

		$atts = shortcode_atts(
			array(
				'title'          => null,
				'comment_label'  => $feedback_options['reply_label'],
				'max_char'       => 100,
				'term_id'        => null,
				'max_char_reply' => 100,
				'seq'            => null,
			),
			$atts
		);

		wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );
		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'eq3f-video-dialog' );
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_style( 'eq3f-feedback-midcopy' );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'eq3f-rating-widget' );
		wp_enqueue_style( 'eq3f-icon-style' );

		wp_localize_script( 'eq3f-feedback-ajax-client-script', 'remsto_ajax_settings', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

		return eq3f_display_mid_copy( $atts );
	}

	/**
	 * Display feedback tabs
	 *
	 * @param type $atts Unused.
	 * @param type $content Unused.
	 * @param type $tag Unused.
	 * @return string Feedback tabs HTML
	 */
	public function enquir3_feedback_tabs( $atts = array(), $content = null, $tag = '' ) {

		wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );
		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_script( 'eq3f-tabs-script' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'eq3f-video-dialog' );
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'eq3f-icon-style' );
		wp_enqueue_style( 'eq3f-rating-widget' );

		wp_localize_script( 'eq3f-feedback-ajax-client-script', 'remsto_ajax_settings', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

		$tab = filter_input( INPUT_GET, 'tab' );
		if ( ! $tab ) {
			$tab = 0; // Promoted tab.
		}
		wp_localize_script( 'eq3f-tabs-script', 'tabs_settings', array( 'tab' => $tab ) );

		$output = '';

    $output .= '<div id="enquir3_feedback_tabs_item_number_container">';
    $output .= '<form id="enquir3_feedback_tabs_item_number" method="get">';
    $items_per_page_options = array(
      5  => '5 items per page',
      10 => '10 items per page',
      20 => '20 items per page',
      50 => '50 items per page',
    );

    // Add a filter for 'items per page'.
    $filter_items_per_page = filter_input( INPUT_GET, 'filter-items-per-page' );
    if ( !isset($filter_items_per_page) || '' === $filter_items_per_page ) {
      $filter_items_per_page = '10';
    }
    $output .= '<select name="filter-items-per-page" class="filter-items-per-page">';
    foreach ( $items_per_page_options as $option_id => $option_title ) {
      if ( intval( $filter_items_per_page ) === $option_id ) {
        $selected = ' selected = "selected"';
      } else {
        $selected = '';
      }
      $output .= '<option value="' . esc_attr( $option_id ) . '"' . esc_attr( $selected ) . '>' . esc_attr( $option_title ) . '</option>';
    }
    $output .= '</select>';
    $output .= '<input type="submit" class="button" value="Go">';
    $output .= '</form>';
    $output .= '</div>';

		$options = eq3f_get_options_new();
		if ( ! empty( $options['feedback_page_labels_intro_text']['value'] ) ) {
			$output .= '<div class="remsto-tabs-intro-text">' . $options['feedback_page_labels_intro_text']['value'] . '</div>';
		}
		$output .= "<div class='remtso-tabs-container clearfix'><ul class='remtso-tabs'>";

		$output .= "<li data-page='page_promoted' class='tab-selected remtso-tab'>Our outstanding feedback</li>";
		$output .= "<li data-page='page_feedback' class='remtso-tab'>More great feedback</li></ul>";

		$output .= "<div id='page_promoted'>";
		$output .= "<div id='promoted_data'>";

		$p0 = filter_input( INPUT_GET, 'p0' );
		if ( ! $p0 ) {
			$p0 = 0;
		}
		// Present the feedback data.
		$output .= $this->remsto_feedback_display_items( 0, 2, $filter_items_per_page, $p0, 0 );
		$output .= '</div></div>';

		$output .= "<div id='page_feedback'>";
		$output .= "<div id='published_dataa'>";
		$p1      = filter_input( INPUT_GET, 'p1' );
		if ( ! $p1 ) {
			$p1 = 0;
		}
		$output .= $this->remsto_feedback_display_items( 0, 1, $filter_items_per_page, $p1, 1 );
		$output .= '</div></div>';

		return $output;
	}

	/**
	 * Display feedback tabs for a listing
	 *
	 * @param type $atts Unused.
	 * @param type $content Unused.
	 * @param type $tag Unused.
	 * @return string Feedback tabs HTML
	 */
	public function enquir3_feedback_listing_tabs( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
			),
			$atts
		);

		wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );
		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_script( 'eq3f-tabs-script' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'eq3f-video-dialog' );
		wp_enqueue_style( 'eq3f-feedback' );
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'eq3f-icon-style' );
		wp_enqueue_style( 'eq3f-rating-widget' );

		wp_localize_script( 'eq3f-feedback-ajax-client-script', 'remsto_ajax_settings', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

		$tab = filter_input( INPUT_GET, 'tab' );
		if ( ! $tab ) {
			$tab = 0; // Promoted tab.
		}
		wp_localize_script( 'eq3f-tabs-script', 'tabs_settings', array( 'tab' => $tab ) );

		$output = '';
    
    $output .= '<div id="enquir3_feedback_tabs_item_number_container">';
    $output .= '<form id="enquir3_feedback_tabs_item_number" method="get">';

    $items_per_page_options = array(
      5  => '5 items per page',
      10 => '10 items per page',
      20 => '20 items per page',
      50 => '50 items per page',
    );

    // Add a filter for 'items per page'.
    $filter_items_per_page = filter_input( INPUT_GET, 'filter-items-per-page' );
    if ( !isset($filter_items_per_page) || '' === $filter_items_per_page ) {
      $filter_items_per_page = '10';
    }
    $output .= '<select name="filter-items-per-page" class="filter-items-per-page">';
    foreach ( $items_per_page_options as $option_id => $option_title ) {
      if ( intval( $filter_items_per_page ) === $option_id ) {
        $selected = ' selected = "selected"';
      } else {
        $selected = '';
      }
      $output .= '<option value="' . esc_attr( $option_id ) . '"' . esc_attr( $selected ) . '>' . esc_attr( $option_title ) . '</option>';
    }
    $output .= '</select>';
    $output .= '<input type="submit" class="button" value="Go">';
    $output .= '</form>';
    $output .= '</div>';

		// Get the listing so we can use the options (if any have been saved).
		$listing_client = new RemstoListingClient( $this->conf );
		$listing		= $listing_client->getListingById( $atts [ 'listing_id' ] );
		if ( isset( $listing->feedback_settings ) && strlen( $listing->feedback_settings ) > 0 ) {
			$options = unserialize($listing->feedback_settings);
		}
		else {
			$options = eq3f_get_options_new();
		}
		
		if ( ! empty( $options['feedback_page_labels_intro_text']['value'] ) ) {
			$output .= '<div class="remsto-tabs-intro-text">' . $options['feedback_page_labels_intro_text']['value'] . '</div>';
		}
		$output .= "<div class='remtso-tabs-container clearfix'><ul class='remtso-tabs'>";

		$output .= "<li data-page='page_promoted' class='tab-selected remtso-tab'>Our outstanding feedback</li>";
		$output .= "<li data-page='page_feedback' class='remtso-tab'>More great feedback</li></ul>";

		$output .= "<div id='page_promoted'>";
		$output .= "<div id='promoted_data'>";

		$p0 = filter_input( INPUT_GET, 'p0' );
		if ( ! $p0 ) {
			$p0 = 0;
		}

    // Present the feedback data.
		$output .= $this->remsto_feedback_display_items( $atts [ 'listing_id' ], 2, $filter_items_per_page, $p0, 0 );
		$output .= '</div></div>';

		$output .= "<div id='page_feedback'>";
		$output .= "<div id='published_dataa'>";
		$p1      = filter_input( INPUT_GET, 'p1' );
		if ( ! $p1 ) {
			$p1 = 0;
		}
		$output .= $this->remsto_feedback_display_items( $atts [ 'listing_id' ], 1, $filter_items_per_page, $p1, 1 );
		$output .= '</div></div>';

		return $output;
	}

	/**
	 * Display feedback performance for a listing
	 *
	 * @param type $atts Unused.
	 * @param type $content Unused.
	 * @param type $tag Unused.
	 * @return string Feedback performance HTML
	 */
	public function enquir3_feedback_listing_performance( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
				'title' => null,
				'excellent_label' => 'Excellent',
				'vgood_label' => 'Very Good',
				'average_label' => 'Average',
				'bad_label' => 'Bad',
				'terrible_label' => 'Terrible',
				'display_numbers' => 'yes',
			),
			$atts
		);

		wp_enqueue_style( 'eq3f-feedback' );
		wp_enqueue_style( 'eq3f-performance-widget' );

    $output = '';

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$performance_data = $fc->getPerformance( 1, $atts [ 'listing_id' ] );
		$output .= '<div class="performance-widget">';

		if ( '' !== $atts [ 'title' ] ) {
			$output .= '<span class="feedback-performance-title">' . esc_attr( $atts [ 'title' ] ) . '</span>';
		}

		//if ( '' !== $instance['widget_title'] ) {
		//	echo '<span class="feedback-performance-title">' . esc_attr( $instance['widget_title'] ) . '</span>';
		//}

		$total = $performance_data->total;
		unset( $performance_data->total );

		foreach ( $performance_data as $name => $performance ) {
			// convert to percentage.
			if ( $total > 0 ) {
				$percentage = round( ( ( $performance / $total ) * 100 ), 0 );
			} else {
				$percentage = 0;
			}

			$bar_data = array(
				'label'       => esc_attr( $atts [ $name.'_label' ] ),
				'name'        => $name,
				'percentage'  => $percentage,
				'performance' => $performance,
				'display_numbers' => esc_attr( $atts [ 'display_numbers' ] ),
			);

			$output .= $this::compile_template(
				EQ3F_PLUGIN_PATH . 'widgets/views/remsto-client-feedback-performance-bar.tpl.php',
				$bar_data );
		}

		$output .= '</div>';

    return $output;
	}

	/**
	 * Display feedback summary for a listing
	 *
	 * @param type $atts Unused.
	 * @param type $content Unused.
	 * @param type $tag Unused.
	 * @return string Feedback summary HTML
	 */
	public function enquir3_feedback_listing_summary( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
				'title' => null,
				'before_label' => 'Feedback received from %n people. ',
				'after_label' => 'would recommend this business.',
			),
			$atts
		);

		wp_enqueue_style( 'eq3f-feedback' );

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$data             = $fc->getFeedbackSummary( 1, $atts [ 'listing_id' ] );

		$total     = 0;
		$recommend = 0;

		foreach ( $data as $row ) {
			$total += $row->total;
			if ( '1' === $row->recommend ) {
				$recommend = $row->total;
			}
		}

		$summary_data = array(
			'total_feedback_count'  => $total,
			'would_recommend_count' => $recommend,
			'title'                 => $atts [ 'title' ],
			'before_label'          => str_replace( '%n', $total, esc_attr( $atts [ 'before_label' ] ) ),
			'after_label'           => str_replace( '%n', $total, esc_attr( $atts [ 'after_label' ] ) ),
		);

		$output = $this::compile_template(
			EQ3F_PLUGIN_PATH . 'widgets/views/feedback-widget-summary.tpl.php',
			$summary_data );


    return $output;
	}

	/**
	 * Display feedback rating for a listing
	 *
	 * @param type $atts Unused.
	 * @param type $content Unused.
	 * @param type $tag Unused.
	 * @return string Feedback rating HTML
	 */
	public function enquir3_feedback_listing_rating( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
				'title' => null,
				'promised_label' => 'Delivered as promised',
				'value_label' => 'Percieved value',
				'helpful_label' => 'Considerate and helpful',
				'communication_label' => 'Deliver a high standard',
				'total_label' => 'Total',
				'display_numbers' => 'yes',
			),
			$atts
		);

		wp_enqueue_style( 'eq3f-feedback' );
		wp_enqueue_style( 'eq3f-rating-widget' );
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$rating_data      = $fc->getRatings( 1,  $atts [ 'listing_id' ]);

		$output = '<div class="rating-widget">';

		if ( '' !== $atts [ 'title' ] ) {
			$output .= '<div class="feedback-rating-title">' . esc_attr( $atts [ 'title' ] ) . '</div>';
		}

		// @todo get this as on option.
		$only_total = false;

		if ( $only_total ) {
			unset( $rating_data->promised );
			unset( $rating_data->value );
			unset( $rating_data->helpful );
			unset( $rating_data->communication );
		}

		foreach ( $rating_data as $key => $rating ) {
			$rating_percent = round( $rating, 0 );

			$variables = array(
				'rating' => $rating_percent,
				'key'    => $key,
				'label'  => esc_attr( $atts[ $key.'_label'] ),
				'display_numbers' => esc_attr( $atts [ 'display_numbers' ] ),
				'stars'  => 5,
			);

			$output .= RemstoModuleFunctions::feedback_rating_display( $variables );
		}

		$output .= '</div>';
		return $output;
	}

  
	public function enquir3_feedback_listing_total_stars( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
				'label' => null,
				'display_numbers' => 'yes',
			),
			$atts
		);

		wp_enqueue_style( 'eq3f-feedback' );
		wp_enqueue_style( 'eq3f-rating-widget' );
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$rating_data      = $fc->getRatings( 1,  $atts [ 'listing_id' ]);

		$rating = $rating_data->total;
    $rating_percent = round( $rating, 0 );

    $variables = array(
      'rating' => $rating_percent,
      'key'    => 'totla',
      'label'  => esc_attr( $atts[ 'label'] ),
      'display_numbers' => esc_attr( $atts [ 'display_numbers' ] ),
      'stars'  => 5,
    );

    $output = RemstoModuleFunctions::feedback_rating_display( $variables );

		return $output;
	}

  public function enquir3_feedback_listing_number_ratings( $atts = array(), $content = null, $tag = '' ) {

		$atts = shortcode_atts(
			array(
				'listing_id' => null,
				'label' => 'Feedback received from %n people. %r would recommend.',
			),
			$atts
		);

		wp_enqueue_style( 'eq3f-feedback' );

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$data             = $fc->getFeedbackSummary( 1, $atts [ 'listing_id' ] );

		$total     = 0;
		$recommend = 0;

		foreach ( $data as $row ) {
			$total += $row->total;
			if ( '1' === $row->recommend ) {
				$recommend = $row->total;
			}
		}

		$summary_data = array(
			'label'          => str_replace ( '%r' , $recommend, str_replace( '%n', $total, esc_attr( $atts [ 'label' ] ) ) ),
		);

		$output = $this::compile_template(
			EQ3F_PLUGIN_PATH . 'widgets/views/feedback-number-ratings.tpl.php',
			$summary_data );


    return $output;
	}

  /**
	 * Get feedback items for a tab.
	 *
	 * @param type $status Status.
	 * @param type $page_number Page number.
	 * @param type $tab Tab (0 = promoted, 1 = published).
	 * @return string contents of a feedback tab
	 */
	public function remsto_feedback_display_items( $listing_id, $status, $items_per_page, $page_number = 0, $tab = 0 ) {

    if($listing_id === 0) {
      $parent_type    = 0;
      $parent_id      = 0;
    }
    else {
      $parent_type    = 1;
      $parent_id      = $listing_id;
    }
    $site_status    = $status;

		$feedback_data = $this->feedback_client->getFeedback(
			$parent_type,
			$parent_id,
			$site_status,
			$items_per_page,
			$page_number,
			null,
			0
		);

		$output = '';

		if ( $feedback_data ) {
			// Present the feedback data.
			foreach ( $feedback_data->data as $feedback ) {
				$feedback_display_data              = $this->format_feedback_for_display( $feedback );
				$feedback_display_data['more_link'] = RemstoModuleFunctions::feedback_more_button(
					array( 'id' => $feedback->id )
				);

				$output .= $this::compile_template(
					EQ3F_PLUGIN_PATH . 'views/feedback-item.tpl.php',
					$feedback_display_data
				);
			}

			// Render pager links.
			if ( $page_number > 0 ) {
				$current_page = $page_number - 1;
			} else {
				$current_page = 0;
			}

      $output .= $this->feedback_pager( $feedback_data->total, $page_number, $tab, $items_per_page );

		}

		return $output;
	}

	/**
	 * Get feedback pager.
	 *
	 * @param int $total Total number of feedback items.
	 * @param int $current_page Current page number.
	 * @param int $tab Tab (0 = promoted, 1 = published).
	 * @return string contents of feedback pager
	 */
	private function feedback_pager( $total, $current_page, $tab, $filter_items_per_page ) {

		$pages = ceil( $total / $filter_items_per_page );

		// dont show pager if only one page exits.
		if ( $pages < 2 ) {
			return '';
		}

		$pager    = "<div class='nav-pager'><ul>";
		$page_url = get_page_link();

		for ( $x = 1; $x <= $pages; $x++ ) {
			$page = $x - 1;
			if ( $page === $current_page ) {
				$current = 'current_page';
			} else {
				$current = '';
			}
			$pager .= "<li class='nav-pager-item $current' data-page='$page'>";
			$pager .= "<a title='Go to page $x' href='$page_url?tab=$tab&amp;p$tab=$page&amp;filter-items-per-page=$filter_items_per_page'>$x</a>";
			$pager .= '</li>';
		}

		$pager .= '</ul></div>';

		return $pager;
	}

	/**
	 * Format feedback data for display
	 *
	 * @param array $feedback Remsto feedback item.
	 * @return array feedback ready to be rendered
	 */
	private function format_feedback_for_display( $feedback ) {

		$feedback_data = (array) $feedback;
		$options       = eq3f_get_options_new();

		$feedback_data['stood_out'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'truncate' => $options['feedback_display_stood_out_length'],
				'label'    => '',
				'value'    => $feedback->stood_out,
				'class'    => 'tpp-field-stood-out',
			)
		);

		$feedback_data['company_comment'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'truncate' => $options['feedback_display_company_comment_length'],
				'label'    => $options['reply_label'],
				'value'    => $feedback->company_comment,
				'class'    => 'tpp-field-company-comment',
			)
		);

		if ( ! empty( $feedback->website ) ) {
			$feedback_data['website'] = RemstoModuleFunctions::iconWebLink( $feedback->website );
		}

		if ( ! empty( $feedback->printable_src ) ) {
			$feedback_data['printable_src'] = RemstoModuleFunctions::printable_src_link(
				$options['remote_url'],
				$feedback->printable_src
			);
		}

		$rating = RemstoModuleFunctions::calculate_rating( $feedback );
		if ( false === $rating ) {
			$rating_summary = 'n/a';
		} else {
			$rating_summary = RemstoModuleFunctions::feedback_rating_display(
				array(
					'rating' => $rating,
					'label'  => '',
					'stars'  => 5,
				)
			);
		}
		$feedback_data['rating_summary'] = $rating_summary;

		$feedback_data['title'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'truncate' => $options['feedback_display_company_comment_length'],
				'label'    => '',
				'value'    => $feedback->title,
				'class'    => 'tpp-field-title',
			)
		);

		$feedback_data['date'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Date:',
				'value' => gmdate( 'd/m/Y', $feedback->created ),
				'class' => 'tpp-field-date',
			)
		);

		$name = array(
			'anon'       => $feedback->anonymous,
			'conc'       => $feedback->concealed,
			'first_name' => $feedback->first_name,
			'last_name'  => $feedback->last_name,
			'town'       => '',
		);

		$author = RemstoModuleFunctions::feedback_name_display( $name );

		$feedback_data['first_name'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Name:',
				'value' => $author,
				'class' => 'tpp-field-first-name',
			)
		);

		if ( ! empty( $feedback->video_testimonial ) ) {
			$feedback_data['video_testimonial'] = RemstoModuleFunctions::processVideoTestimonial(
				$feedback->id,
				$feedback->video_testimonial
			);
		}

		$feedback_data['position'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Job Title:',
				'value' => $feedback->position,
				'class' => 'tpp-field-position',
			)
		);

		$feedback_data['business_category'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Sector:',
				'value' => $feedback->business_category,
				'class' => 'tpp-field-category',
			)
		);

		$feedback_data['company'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Company:',
				'value' => $feedback->company_name,
				'class' => 'tpp-field-company_name',
			)
		);

		$feedback_data['town'] = RemstoModuleFunctions::feedback_text_value(
			array(
				'label' => 'Town:',
				'value' => $feedback->town,
				'class' => 'tpp-field-town',
			)
		);

		if ( isset( $feedback->taxonomy_all ) ) {
			$services = array();
			foreach ( $feedback->taxonomy_all as $tid => $taxonomy ) {
				$taxonomy = $this->feedback_client->getTaxonomy( $tid );
				if ( ! $taxonomy ) {
					continue;
				}
				if ( RemstoClientTaxonomyDisplayOptions::isOptionSet(
					$taxonomy->display_options,
					RemstoClientTaxonomyDisplayOptions::LISTSERVICE
				) ) {
					foreach ( $feedback->taxonomy_all->$tid as $term_id ) {
						if ( isset( $taxonomy->terms->$term_id ) ) {
							$term       = $taxonomy->terms->$term_id;
							$services[] = $term->title;
						}
					}
				}
			}

			$services_text             = implode( ', ', $services );
			$feedback_data['services'] = RemstoModuleFunctions::feedback_icon_text_value(
				array(
					'icon_class' => 'icon-services',
					'value'      => "<b>Service(s) Provided:</b> $services_text ",
				)
			);
		}

		return $feedback_data;
	}

	/**
	 * Register plug-ins widgets.
	 *
	 * @return void
	 */
	public function remsto_feedback_create_widgets() {
		register_widget( 'Eq3f_Widget_Summary' );
		register_widget( 'Eq3f_Widget_Performance' );
		register_widget( 'Eq3f_Widget_Rating' );
		register_widget( 'Eq3f_Latest_Feedback_Widget' );
	}

	/**
	 * Form for enter/edit feedback.
	 *
	 * @param array $atts Attributes.
	 * @return str Some text
	 */
	public function enquir3_feedback_entry_form_new( $atts ) {
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_script( 'eq3f-feedback_form' );
		wp_enqueue_script( 'eq3f-forms' );
		$options = eq3f_get_options_new();

		$display_form = true;
		$post         = filter_input_array( INPUT_POST );

		$validation_errors = array();
		if ( ! empty( $post ) ) {
			$feedback_data                      = $post;
			$feedback_data['anonymous']         = filter_input( INPUT_POST, 'anonymous' );
			$feedback_data['company_comment']   = filter_input( INPUT_POST, 'company_comment' );
			$feedback_data['services_provided'] = array();
			$feedback_data['tid']               = 0;
			$this->process_taxonomy( $feedback_data );

			// Attempt to do some validation.
			if ( empty( $feedback_data['stood_out'] ) ) {
				$validation_errors[] = 'The field "' . $options['feedback_stood_out_label'] . '" is required.';
			}
			if ( '1' !== $feedback_data['anonymous'] ) {
				if ( empty( $feedback_data['first_name'] ) ) {
					$validation_errors[] = 'The field "' . $options['feedback_first_name_label'] . '" is required.';
				}
				if ( empty( $feedback_data['last_name'] ) ) {
					$validation_errors[] = 'The field "' . $options['feedback_last_name_label'] . '" is required.';
				}
				if ( empty( $feedback_data['email'] ) ) {
					$validation_errors[] = 'The field "' . $options['feedback_email_label'] . '" is required.';
				}
			}
			for ( $i = 0; $i < 4; $i++ ) {
				if ( '1' === $options['feedback_extra_rating_questions_mandatory'][ $i ] ) {
					if ( empty( $feedback_data[ 'extra_worded_question_' . ( $i + 1 ) ] ) ) {
						$validation_errors[] = 'The field "' . $options['feedback_extra_worded_questions_label'][ $i ] . '" is required.';
					}
				}
			}
			if ( empty( $validation_errors ) ) {
				$check_captcha = apply_filters( 'gglcptch_verify_recaptcha', true, 'string', 'enquir3_feedback_form' );
				// Check reCaptcha result.
				if ( true === $check_captcha ) {
					$display_form = false;

					if ( isset( $feedback_data['created'] ) ) {
						$created_date = str_replace( '/', '-', $feedback_data['created']['date'] ); // make sure date use d/m/y.
						$created_time = $feedback_data['created']['time'];
						$new_created  = strtotime( "$created_date $created_time" );
						// if entered date/time not valid leave created unchanged.
						if ( $new_created ) {
							$feedback_data['created'] = $new_created;
						} else {
							$feedback_data['created'] = $feedback_data['old_created'];
						}
						unset( $feedback_data['old_created'] );
					} else {
						$feedback_data['created'] = time();
					}

					// Site status.
					if ( ! isset( $feedback_data['site_status'] ) ) {
						$feedback_data['site_status'] = array(
							$options['client_id'] => 'on' === $options['feedback_auto_publish'] ? 1 : 0,
						);
					}

					// Printable data.
					if ( isset( $_FILES['files']['tmp_name']['feedback_printable'] ) &&
						'' !== $_FILES['files']['tmp_name']['feedback_printable'] ) {
						$feedback_data['printable_name'] = wp_unslash( $_FILES['files']['name']['feedback_printable'] );
						$feedback_printable_file         = wp_unslash( $_FILES['files']['tmp_name']['feedback_printable'] );
						$feedback_printable_name         = wp_unslash( $_FILES['files']['name']['feedback_printable'] );
					} else {
						$feedback_printable_file = null;
						$feedback_printable_name = null;
					}
					if ( isset( $post['edit-submit-feedback-printable'] ) ) {
						$feedback_data['printable_delete'] = 1;
					}

					// Image data.
					if ( isset( $_FILES['files']['tmp_name']['feedback_image'] ) &&
						'' !== $_FILES['files']['tmp_name']['feedback_image'] ) {
						$feedback_data['image_name'] = wp_unslash( $_FILES['files']['name']['feedback_image'] );
						$feedback_image_file         = wp_unslash( $_FILES['files']['tmp_name']['feedback_image'] );
						$feedback_image_name         = wp_unslash( $_FILES['files']['name']['feedback_image'] );
					} else {
						$feedback_image_file = null;
						$feedback_image_name = null;
					}
					if ( isset( $post['edit-submit-feedback-image'] ) ) {
						$feedback_data['image_delete'] = 1;
					}

					unset( $feedback_data['op'] );
					if ( ! empty( $feedback_data['feedback_id'] ) ) {
						$this->feedback_client->updateFeedback(
							$feedback_data,
							$feedback_printable_file,
							$feedback_printable_name,
							$feedback_image_file,
							$feedback_image_name
						);
						// Start referral process.
						if ( isset( $feedback_data['start_referral_process'] ) ) {
							do_action( 'eq3f_feedback_edited', $feedback_data['feedback_id'] );
						}
					} else {
						$this->feedback_client->addFeedback(
							$feedback_data,
							$feedback_printable_file,
							$feedback_printable_name,
							$feedback_image_file,
							$feedback_image_name
						);
					}

					$page = filter_input( INPUT_GET, 'page' );
					if ( 'feedback-management' !== $page ) {
						$email_from    = $options['feedback_email_notification_from'];
						$email_to      = $options['feedback_email_notification_to'];
						$email_subject = $options['feedback_email_notification_subject'] . 'EQ3_From:' . $email_from;

						// Convert taxonomy into a suitable array.
						if ( isset( $feedback_data['taxonomy'] ) ) {
							$taxonomy = array();
							foreach ( $feedback_data['taxonomy'] as $vocabulary_id => $terms ) {
								$taxonomy[ $vocabulary_id ] = array();
								foreach ( $terms as $term_id ) {
									if ( 0 !== $term_id ) {
										$taxonomy[ $vocabulary_id ][] = $term_id;
									}
								}
							}
							$feedback_data['taxonomy'] = $taxonomy;
						}

						// Service taxonomy.
						$tax_service                           = $this->feedback_client->getTaxonomy( $options['feedback_service_taxonomy'] );
						$feedback_data['service_taxonomy_all'] = json_decode( wp_json_encode( $tax_service ), true ); // casting does not convert nested objects.

						// Additional Taxonomy.
						$feedback_data['feedback_additional_taxonomy_all'] = array();
						foreach ( $options['feedback_additional_taxonomy'] as $vocabulary_id ) {
							$tax_additional                                      = $this->feedback_client->getTaxonomy( $vocabulary_id );
							$feedback_data['feedback_additional_taxonomy_all'][] = json_decode( wp_json_encode( $tax_additional ), true );
						}

						$feedback_data['options'] = $options;

						if ( ! empty( $email_to ) ) {

							$email_body = $this::compile_template(
								EQ3F_PLUGIN_PATH . 'views/remsto-notification-feedback-html-mail.tpl.php',
								$feedback_data
							);
							$headers    = array(
								'Content-Type: text/html; charset=UTF-8',
								'From:' . $email_from,
							);
							wp_mail( $email_to, $email_subject, $email_body, $headers );
						}
					}

					if ( 'feedback-management' === $page ) {
						print "<div class='notice notice-success is-dismissible'><p>Feedback updated</div></p>";
						$this->remsto_feedback_management_render_table();
						exit();
					} else {
						$thankyou_page = get_post( $options['thankyou_page_id'] );
						if ( $thankyou_page ) {
								return $thankyou_page->post_content;
						} else {
							return '<div>Thank you for your feedback!</div><a href="/feedback">View feedback</a>';
						}
					}
				}
			}
		}
		if ( $display_form ) {
			$fid = filter_input( INPUT_GET, 'fid' );
			if ( $fid ) {
				$feedback_data = (array) $this->feedback_client->getFeedbackItem( 0, 0, $fid );
				// Moderation Section.
				wp_enqueue_script( 'jquery-ui-core' );
				wp_enqueue_script( 'jquery-ui-dialog' );
				wp_enqueue_style( 'wp-jquery-ui-dialog' );
				wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );
				wp_enqueue_script( 'eq3f-feedback-ajax-buttons' );
				wp_localize_script(
					'eq3f-feedback-ajax-client-script',
					'remsto_ajax_settings',
					array( 'ajax_url' => admin_url( 'admin-ajax.php' ) )
				);

				$feedback_data['created_date'] = gmdate( 'd/m/Y', $feedback_data['created'] );
				$feedback_data['created_time'] = gmdate( 'h:i', $feedback_data['created'] );
				// Display referral checkbox.
				if ( is_plugin_active( 'enquir3-referrals/enquir3-referrals.php' ) ) {
					$feedback_data['referral_element'] = Enquir3_Referral::get_referral_form_element( $feedback_data );
				}
				$feedback_data['display_company_comment_length'] = $options['feedback_display_company_comment_length'];

				$feedback_data['moderation_section'] = true;
				// }
			} else {
				if ( ! isset( $feedback_data ) ) {
					$feedback_data = array(
						'recommend'            => '1',
						'rating_promised'      => '100',
						'rating_value'         => '100',
						'rating_helpful'       => '100',
						'rating_communication' => '100',
						'anonymous'            => '0',
						'concealed'            => '0',
						'staff_service'        => '',
						'taxonomy'             => null,
					);
				}
				$feedback_data['moderation_section'] = false;
			}

			// Convert taxonomy into a suitable array.
			if ( isset( $feedback_data['taxonomy'] ) ) {
				$taxonomy = array();
				foreach ( $feedback_data['taxonomy'] as $vocabulary_id => $terms ) {
					$taxonomy[ $vocabulary_id ] = array();
					foreach ( $terms as $term_id ) {
						if ( 0 !== $term_id ) {
							$taxonomy[ $vocabulary_id ][] = $term_id;
						}
					}
				}
				$feedback_data['taxonomy'] = $taxonomy;
			}

			// Staff Service taxonomy.
			if ( $options['feedback_staff_service_taxonomy'] > 0 ) {
				$tax_staff_service                           = $this->feedback_client->getTaxonomy( $options['feedback_staff_service_taxonomy'] );
				$feedback_data['staff_service_taxonomy_all'] = json_decode( wp_json_encode( $tax_staff_service ), true ); // casting does not convert nested objects.
			}
			// Service taxonomy.
			$tax_service                           = $this->feedback_client->getTaxonomy( $options['feedback_service_taxonomy'] );
			$feedback_data['service_taxonomy_all'] = json_decode( wp_json_encode( $tax_service ), true ); // casting does not convert nested objects.

			// Additional Taxonomy.
			$feedback_data['feedback_additional_taxonomy_all'] = array();
			foreach ( $options['feedback_additional_taxonomy'] as $vocabulary_id ) {
				$tax_additional                                      = $this->feedback_client->getTaxonomy( $vocabulary_id );
				$feedback_data['feedback_additional_taxonomy_all'][] = json_decode( wp_json_encode( $tax_additional ), true );
			}

			// Anonymous.
			if ( '1' === $feedback_data['anonymous'] ) {
				$feedback_data['anonymous_checked'] = 'checked';
			} else {
				$feedback_data['anonymous_checked'] = '';
			}

			// Concealed.
			if ( isset( $feedback_data['concealed'] ) && '1' === $feedback_data['concealed'] ) {
				$feedback_data['concealed_checked'] = 'checked';
			} else {
				$feedback_data['concealed_checked'] = '';
			}

			// Length of company comment.
			$feedback_data['feedback_display_company_comment_length'] = $options['feedback_display_company_comment_length'];

			// reCaptcha.
			$feedback_data['captcha'] = apply_filters( 'gglcptch_display_recaptcha', '', 'enquir3_feedback_form' );
			if ( isset( $check_captcha ) ) {
				$validation_errors[] = $check_captcha;
			}

			// Set up error messages.
			if ( ! empty( $validation_errors ) ) {
				$feedback_data['error_message'] = $validation_errors;
			}

			// Submit button.
			$feedback_data['feedback_add_text'] = $options['feedback_add_text'];

			$feedback_data['options'] = $options;
			wp_enqueue_style( 'eq3f-feedback-admin' );

			return $this::compile_template(
				EQ3F_PLUGIN_PATH . 'views/remsto-feedback-form-new.tpl.php',
				$feedback_data
			);
		}
	}

	/**
	 * Process taxomomy data from feedback.
	 *
	 * @param array $feedback_data Feedback.
	 * @return void
	 */
	private function process_taxonomy( &$feedback_data ) {

		$options                 = eq3f_get_options_new();
		$services_vocabulary_id  = $options['feedback_service_taxonomy'];
		$additional_taxonomies   = $options['feedback_additional_taxonomy'];
		$additional_taxonomies[] = $services_vocabulary_id;

		// Additional taxonomies.
		$taxonomy_data                            = array();
		$taxonomy_data[ $services_vocabulary_id ] = array();
		foreach ( $additional_taxonomies as $tid ) {
			if ( isset( $feedback_data[ "vocabulary_$tid" ] ) && is_array( $feedback_data[ "vocabulary_$tid" ] ) ) {
				$t = $feedback_data[ "vocabulary_$tid" ];
				foreach ( $t as $term_id => $term ) {
					$taxonomy_data[ $tid ][ $term_id ] = (int) $term;
				}
				unset( $feedback_data[ "vocabulary_$tid" ] );
			}
		}
		$feedback_data['taxonomy']          = $taxonomy_data;
		$feedback_data['services_provided'] = $taxonomy_data[ $services_vocabulary_id ];
	}

	/**
	 * Add items to the menus.
	 *
	 * @return void
	 */
	public function add_menu_items() {
		add_menu_page(
			'Enquir3 Feedback Management',
			'Enquir3 Feedback Management',
			'enquir3_feedback_moderate',
			'feedback-management',
			array( $this, 'remsto_feedback_management_render_page' ),
			'dashicons-format-chat'
		);
    add_submenu_page( 
        'feedback-management',
        'Enquir3 Feedback Management', 
        'View Feedback', 
        'enquir3_feedback_moderate', 
        'feedback-management',
        array( $this, 'remsto_feedback_management_render_page' )
    );
    add_submenu_page( 
        'feedback-management',
        'Export Feedback', 
        'Export Feedback', 
        'enquir3_feedback_moderate', 
        'feedback-export',
        array( $this, 'remsto_feedback_export_render_page' )
    );
		add_menu_page(
			'Enquir3 Taxonomy',
			'Enquir3 Taxonomy',
			'enquir3_feedback_taxonomy_configure',
			'feedback-taxonomy-management',
			array( $this, 'taxonomy_maintenance_page' ),
			'dashicons-tag'
		);
	}

	/**
	 * Add items to admin init.
	 *
	 * @return void
	 */
	public function add_admin_init() {
    $page = filter_input( INPUT_GET, 'page' );
    if ( 'feedback-export' === $page ) {
      $format = filter_input( INPUT_GET, 'format' );
      if ( !isset( $format ) ) {
        $format = 'summary';
      }
      $validate = $this->remsto_feedback_export_validate();

      $from_date = $validate[ 'from_date' ];
      $to_date = $validate[ 'to_date' ];

      if ( 'Correct' === $validate[ 'message' ] ) {
        $this->remsto_feedback_export_submit($from_date, $to_date, $format);
      }
    }
  }

	/**
	 * Render feedback management page.
	 *
	 * @return void
	 */
	public function remsto_feedback_management_render_page( $action = '', $fids = array()) {
    if ( '' == $action ) {
      $action = filter_input( INPUT_GET, 'action' );
    }
		switch ( $action ) {
			case 'edit':
				print do_shortcode( '[enquir3-feedback-form-new]' );
				break;
			case 'delete':
        if ( count( $fids ) == 0 ) {
          $fid = filter_input( INPUT_GET, 'fid' );
          $fids = array( $fid );
        }
        $comment = '';
        foreach( $fids as $fid ) {
          $feedback_item = $this->feedback_client->getFeedbackItem( 0, 0, $fid );
          if ( $feedback_item ) {
            $comment .= '<p>'.$feedback_item->stood_out.'</p>';
          }
        }
				$hidden   = array(
					array(
						'name'  => 'fids',
						'value' => serialize( $fids ),
					),
				);
				$settings = array(
					'link'    => esc_url( admin_url( 'admin-post.php' ) ),
					'action'  => 'feedback_delete_form_submit',
					'hidden'  => $hidden,
					'message' => "Are you sure you wish to delete <br> $comment",
				);

				$this::render_template(
					EQ3F_PLUGIN_PATH . 'views/feedback-confirmation-form.tpl.php',
					$settings
				);
        exit();
				break;
			default:
				// Create an instance of our package class...
				$test_list_table = new EQ3F_Feedback_Management_Table( $this );
				// Fetch, prepare, sort, and filter our data...
				$test_list_table->prepare_items();
				?>
		<div class="wrap">

			<div id="icon-users" class="icon32"><br /></div>
			<h2>Feedback Management</h2>

			<!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
			<form id="feedback-filter" method="get">
				<input type="hidden" name="page" value="<?php echo filter_input( INPUT_GET, 'page' ); ?>" />
				<?php $test_list_table->display(); ?>
			</form>

		</div>
				<?php
		}
	}

	/**
	 * Render Feedback management table.
	 *
	 * @return void
	 */
	public function remsto_feedback_management_render_table() {
		$test_list_table = new EQ3F_Feedback_Management_Table( $this->feedback_client );
		// Fetch, prepare, sort, and filter our data...
		$test_list_table->prepare_items();
		?>
		<div class="wrap">

			<div id="icon-users" class="icon32"><br /></div>
			<h2>Feedback Management</h2>

			<!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
			<form id="feedback-filter" method="get">
				<input type="hidden" name="page" value="<?php echo filter_input( INPUT_GET, 'page' ); ?>" />
				<?php $test_list_table->display(); ?>
			</form>

		</div>
		<?php
	}

	/**
	 * Render Feedback export table.
	 *
	 * @return void
	 */
  public function remsto_feedback_export_render_page( $action = '', $fids = array()) {
    $format = filter_input( INPUT_GET, 'format' );
    if ( !isset( $format ) ) {
      $format = 'summary';
    }
    $validate = $this->remsto_feedback_export_validate();

    $from_date = $validate[ 'from_date' ];
    $to_date = $validate[ 'to_date' ];

    if ( 'Correct' === $validate[ 'message' ] ) {
      // Actually do the export.
      //$this->remsto_feedback_export_submit($from_date, $to_date, $format);
      
    }
    else {
      $message = $validate[ 'message' ];
    }
				?>
		<div class="wrap">

			<div id="icon-users" class="icon32"><br /></div>
			<h2>Feedback Export</h2>
      <?php if ( isset( $message ) ) { ?>
      <div class='error_message'><?php print $message; ?></div>
      <?php } ?>
			<form id="feedback-export" method="get">
				<input type="hidden" name="page" value="<?php print filter_input( INPUT_GET, 'page' ); ?>" />
        <label for="from_date">From date:</label>
        <input type="date" id="from_date" name="from_date"
               value="<?php print $from_date; ?>"
               min="2018-01-01" max="2024-12-31">
        <br/>
        <label for="to_date">To date:</label>
        <input type="date" id="to_date" name="to_date"
               value="<?php print $to_date; ?>"
               min="2018-01-01" max="2024-12-31">
        <br/>
        <label for="edit-format">Format</label><br>
        <div id="edit-format">
        <input type="radio" id="summary" name="format" value="summary" <?php if ( 'summary' === $format ) { ?>checked<?php } ?>
        <label for="summary">Summary</label><br>
        <input type="radio" id="detailed" name="format" value="detailed" <?php if ( 'detailed' === $format ) { ?>checked<?php } ?>
        <label for="detailed">Detailed</label><br>
        </div>
        <br/>
        <input type="submit" id="edit-submit" name="op" value="Submit" class="form-submit">
      </form>
		</div>
				<?php
  }

	/**
	 * Validate the feedback eport form.
	 *
	 * @return array error mesage.
	 */
  private function remsto_feedback_export_validate() {
    $from_date = filter_input( INPUT_GET, 'from_date' );
    $to_date = filter_input( INPUT_GET, 'to_date' );
    if ( !isset ( $from_date ) || !isset ( $to_date ) ) {
      $now = date( 'Y-m-d', time()) ;
      $from_date = strtotime( $from_date );
      $to_date = strtotime( $to_date );
      return array(
        'from_date' => $now,
        'to_date' => $now,
        'message' => '',
      );
    }

    if ($to_date < $from_date) {
      return array(
        'from_date' => $from_date,
        'to_date' => $to_date,
        'message' => 'To date must be after From date',
      );
    }
    else {
      return array(
        'from_date' => $from_date,
        'to_date' => $to_date,
        'message' => 'Correct',
      );
    }
  }


	/**
	 * Process feedback export.
   *
   * @param string $from_date
   * @param string $to_date
   * @param string $format
   * 
   */
  private function remsto_feedback_export_submit($from, $to, $format) {
    $from_date = strtotime( $from . ' 00:00');
    $to_date = strtotime( $to . ' 23:59');

    $feedback_settings = eq3f_get_options_new();

    $services_vocabulary_id = $feedback_settings['feedback_service_taxonomy'];
    $additional_vocabularies = $feedback_settings['feedback_additional_taxonomy'];
    $display_optional = $feedback_settings['feedback_display_optional_fields'];

    $labels = $this->remsto_client_feedback_get_form_labels($feedback_settings);
    $weights = $this->remsto_client_feedback_get_form_weights($additional_vocabularies, $feedback_settings);
    $weights['more_services_group'][$services_vocabulary_id]=$weights['services'];
    foreach($weights as $key1=>$weight1) {
      if(is_array($weight1)) {
        foreach($weight1 as $key2=>$weight2) {
          $weights[$key1][$key2]*=10;
        }
      }
      else {
        $weights[$key1]*=10;
      }
    }

    $data = $this->feedback_client->getFeedbackExport(0, 0, $from_date, $to_date);

    $feedback = $data->data;

    // clean output buffer.
    //ob_end_clean();

    header('Content-Type: text/csv');
    header('Content-Disposition: attachment; filename="export.csv"');

    $handle = fopen("php://output", 'wb');

    // Taxonomy
    $additional_vocabularies[$services_vocabulary_id] = $services_vocabulary_id;
    $taxonomy_details = array();
    foreach ($additional_vocabularies as $tid) {
      if ($tid > 0) {
        $td = $this->feedback_client->getTaxonomy($tid);
        $taxonomy_details[$tid] = json_decode(json_encode($td->terms), true);
        $taxonomy_details[$tid]['title'] = $td->title;
      }
    }
    if ($format === '0') {
      $header = $this->remsto_client_feedback_admin_csv_export_summary_header();
    }
    else {
      $header = $this->remsto_client_feedback_admin_csv_export_detail_header(
        $labels,
        $weights,
        $additional_vocabularies,
        $taxonomy_details,
        $display_optional);
    }
    fputcsv($handle, $header);

    foreach ($feedback as $fb) {

      if ($format === '0') {
        $row = $this->remsto_client_feedback_admin_csv_export_summary_row($fb);
      }
      else {
        $row = $this->remsto_client_feedback_admin_csv_export_detail_row(
          $fb,
          $labels,
          $weights,
          $additional_vocabularies,
          $taxonomy_details,
          $display_optional);
      }

      fputcsv($handle, $row);
    }

    fclose($handle);
    exit();
  }

	/**
	 * Display taxonomy selections.
	 *
	 * @param int      $vocabulary_id Taxonomy vocabulary ID.
	 * @param stdClass $feedback_taxonomy Contains all selected taxonomy terms.
	 * @return type
	 */
	private function display_taxonomy( $vocabulary_id, $feedback_taxonomy = null ) {

		if ( ! $feedback_taxonomy ) {
			$feedback_taxonomy = new stdClass();
		}

		$tax       = $this->feedback_client->getTaxonomy( $vocabulary_id );
		$tax_array = json_decode( wp_json_encode( $tax ), true ); // casting does not convert nested objects.

		// If data found display exiting taxonomy settings.
		if ( isset( $feedback_taxonomy->$vocabulary_id ) ) {
			$current_selections = (array) $feedback_taxonomy->$vocabulary_id;
			foreach ( $tax_array['terms'] as $term ) {
				if ( in_array( $term['term_id'], $current_selections, true ) ) {
					$tax_array['terms'][ $term['term_id'] ]['checked'] = 'checked';
				} else {
					$tax_array['terms'][ $term['term_id'] ]['checked'] = '';
				}
			}
		}
		return $this::compile_template(
			EQ3F_PLUGIN_PATH . 'views/taxonomy-form-element.tpl.php',
			$tax_array
		);
	}

	/**
	 * Render taxonomy maintenance page.
	 */
	public function taxonomy_maintenance_page() {
		if ( ! current_user_can( 'enquir3_feedback_moderate' ) ) {
			wp_die( 'Not Authorised' );
		}

		wp_enqueue_style( 'eq3f-feedback-admin' );

		$action = filter_input( INPUT_GET, 'action' );
		switch ( $action ) {
			case 'edit':
				$tid      = filter_input( INPUT_GET, 'tid' );
				$editform = new EQ3F_Feedback_Taxonomy_Edit_Form( $this->feedback_client, $tid );
				$editform->display();
				break;
			case 'terms':
				// Display taxonomy terms for specified vocabuary.
				$tid             = filter_input( INPUT_GET, 'tid' );
				$term_list_table = new EQ3F_Feedback_Taxonomy_Term_List( $this->feedback_client, $tid );
				$term_list_table->prepare_items();
				$link = admin_url( "admin.php?page=feedback-taxonomy-management&action=term-edit&tid=$tid" );
				echo '<div class="wrap">';
				echo '<h1 class="wp-heading-inline">Taxonomy Terms</h1>';
				echo "<a href='" . esc_url( $link ) . "' class='page-title-action'>Add New</a>";
				echo '<hr class="wp-header-end">';
				$term_list_table->display();
				echo '</div>';
				break;
			case 'term-edit':
				$tid           = filter_input( INPUT_GET, 'tid' );
				$ttid          = filter_input( INPUT_GET, 'ttid' );
				$term_editform = new EQ3F_Feedback_Taxonomy_Term_Edit_Form( $this->feedback_client, $tid, $ttid );
				$term_editform->display();
				break;
			case 'tdel':
				$term_id = filter_input( INPUT_GET, 'ttid' );
				$term    = $this->feedback_client->getTaxonomyTermById( $term_id );
				if ( $term ) {
					$term_title = $term->title;
				} else {
					$term_title = '';
				}
				$hidden   = array(
					array(
						'name'  => 'tid',
						'value' => filter_input( INPUT_GET, 'tid' ),
					),
					array(
						'name'  => 'term_id',
						'value' => $term_id,
					),
				);
				$settings = array(
					'link'    => esc_url( admin_url( 'admin-post.php' ) ),
					'action'  => 'taxonomy_term_delete_form_submit',
					'hidden'  => $hidden,
					'message' => "Are you sure you wish to delete term - $term_title ?",
				);
				$this::render_template(
					EQ3F_PLUGIN_PATH . 'views/feedback-confirmation-form.tpl.php',
					$settings
				);
				break;
			default:
				$term_list_table = new EQ3F_Feedback_Taxonomy_List( $this->feedback_client );
				$term_list_table->prepare_items();
				$link = admin_url( 'admin.php?page=feedback-taxonomy-management&action=edit' );
				echo '<div class="wrap">';
				echo '<h1 class="wp-heading-inline">Taxonomy</h1>';
				echo "<a href='" . esc_url( $link ) . "' class='page-title-action'>Add New</a>";
				echo '<hr class="wp-header-end">';
				$term_list_table->display();
				echo '</div>';
		}
	}

	/**
	 * Process taxonomy form submit.
	 * Edit/Add Taxonomy
	 */
	public function taxonomy_form_submit() {

		try {

			$op = filter_input( INPUT_POST, 'op' );
			if ( 'Cancel' !== $op ) {
				$title = filter_input( INPUT_POST, 'taxonomy_name' );
				$tid   = filter_input( INPUT_POST, 'tid' );
				$test  = filter_input( INPUT_POST, 'display_options', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );

				if ( $test ) {
					$display_options = RemstoClientTaxonomyDisplayOptions::selectedOptionsToStore( $test );
				} else {
					$display_options = 0;
				}
				$taxonomy = array(
					'title'           => $title,
					'status'          => 1,
					'display_options' => $display_options,
				);

				if ( '' !== $tid ) {
					$taxonomy['tid'] = $tid;
				}

				$tid = $this->feedback_client->addTaxonomy( $taxonomy );
			}
			wp_safe_redirect( admin_url( 'admin.php?page=feedback-taxonomy-management' ) );
			exit();
		} catch ( Exception $ex ) {
			print esc_attr( $ex->getMessage() );
		}
	}

	/**
	 * Process taxonomy term form.
	 */
	public function taxonomy_term_form_submit() {

		try {

			$op  = filter_input( INPUT_POST, 'op' );
			$tid = filter_input( INPUT_POST, 'tid' );
			if ( 'Cancel' !== $op ) {
				$title      = filter_input( INPUT_POST, 'taxonomy_term_name' );
				$position   = filter_input( INPUT_POST, 'taxonomy_term_position' );
				$aka        = filter_input( INPUT_POST, 'taxonomy_term_aka' );
				$ttid       = filter_input( INPUT_POST, 'ttid' );

				$term = array(
					'title'          => $title,
					'position'       => $position,
					'pseudonym'      => $aka,
					'status'         => 1,
					'tid'            => $tid,
					'parent_term_id' => 0,
				);

				if ( '' !== $ttid ) {
					$term['term_id'] = $ttid;
				}

				$this->feedback_client->createTaxonomyTerm( $tid, $term );
			}
			wp_safe_redirect( admin_url( "admin.php?page=feedback-taxonomy-management&action=terms&tid=$tid" ) );
			exit();
		} catch ( Exception $ex ) {
			print esc_attr( $ex->getMessage() );
		}
	}

	/**
	 * Handle term delete.
	 */
	public function taxonomy_term_delete_form_submit() {

		try {

			$op  = filter_input( INPUT_POST, 'op' );
			$tid = filter_input( INPUT_POST, 'tid' );

			if ( 'Yes' === $op ) {
				$term_id = filter_input( INPUT_POST, 'term_id' );
				$rc      = $this->feedback_client->deleteTaxonomyTerm( $tid, $term_id );
				if ( ! isset( $rc ) ) {
					$action_status = '&action_status=' . EQ3F_TAXONOMY_TERM_DELETE_FAILURE;
				} else {
					$action_status = '&action_status=' . EQ3F_TAXONOMY_TERM_DELETE_SUCCESS;
				}
			}

			wp_safe_redirect( admin_url( "admin.php?page=feedback-taxonomy-management&action=terms&tid=$tid$action_status" ) );
			exit();
		} catch ( Exception $ex ) {
			print esc_attr( $ex->getMessage() );
		}
	}

	/**
	 * Delete feedback
	 */
	public function feedback_delete_form_submit() {
		try {

			$op = filter_input( INPUT_POST, 'op' );
			if ( 'Yes' === $op ) {
        $fids = unserialize( filter_input( INPUT_POST, 'fids' ) );
        foreach ( $fids as $fid) {
          $this->feedback_client->deleteFeedbackItem( $fid );
        }
			}

			wp_safe_redirect( admin_url( 'admin.php?page=feedback-management' ) );
			exit();
		} catch ( Exception $ex ) {
			print esc_attr( $ex->getMessage() );
		}
	}

	/**
	 * Add capabilities.
	 */
	public static function add_capability() {
		$role = get_role( 'administrator' );
		$role->add_cap( 'enquir3_feedback_moderate' );
		$role->add_cap( 'enquir3_feedback_taxonomy_configure' );
	}


	/**
	 * Add capabilities.
	 */
	public static function remove_capability() {
		$role = get_role( 'administrator' );
		$role->remove_cap( 'enquir3_feedback_moderate' );
		$role->remove_cap( 'enquir3_feedback_moderate' );
	}


	/**
	 * Display messages.
	 */
	public function display_messages() {
		$action_status = filter_input( INPUT_GET, 'action_status' );
		if ( isset( $action_status ) ) {
			switch ( $action_status ) {
				case EQ3F_TAXONOMY_TERM_DELETE_FAILURE:
					add_action( 'admin_notices', array( $this, 'taxonomy_term_delete_failure' ) );
					break;
				case EQ3F_TAXONOMY_TERM_DELETE_SUCCESS:
					add_action( 'admin_notices', array( $this, 'taxonomy_term_delete_success' ) );
					break;
				default:
					break;
			}
		}
	}

	/**
	 * Display message on successful deletion of taxonomy term.
	 */
	public function taxonomy_term_delete_success() {
		?>
		<div class="updated notice">
			<p>Taxonomy term successfully deleted.</p>
		</div>
		<?php
	}

	/**
	 * Display message on unsuccessful deletion of taxonomy term.
	 */
	public function taxonomy_term_delete_failure() {
		?>
		<div class="error notice">
			<p>Taxonomy term not deleted. It is probably in use.</p>
		</div>
		<?php
	}

	/**
	 * Add ability to add reCaptcha to feedback form.
	 */
	public function add_custom_recaptcha_forms() {
		$forms['enquir3_feedback_form'] = array(
			'form_name' => 'Enquir3 Feedback Entry Form',
		);
		return $forms;
	}

	/**
	 * Compile output from a template and variables.
	 *
	 * @param string $template_path path to template file.
	 * @param array  $vars array of replacement strings.
	 * @return string HTML output
	 */
	public static function compile_template( $template_path, $vars ) {
		// Start output buffering.
		ob_start();

		// Include the template file.
		include $template_path;

		// End buffering and return its contents.
		return ob_get_clean();
	}

	/**
	 * Compile output from a template and variables.
	 *
	 * @param string $template_path path to template file.
	 * @param array  $vars array of replacement strings.
	 */
	public static function render_template( $template_path, $vars ) {
		$output       = self::compile_template( $template_path, $vars );
		$allowed_html = array(
			'form'     => array(
				'action'  => array(),
				'enctype' => array(),
				'method'  => array(),
			),
			'input'    => array(
				'id'        => array(),
				'class'     => array(),
				'type'      => array(),
				'name'      => array(),
				'value'     => array(),
				'size'      => array(),
				'maxlength' => array(),
				'checked'   => array(),
			),
			'label'    => array(
				'for' => array(),
			),
			'div'      => array(
				'id'      => array(),
				'class'   => array(),
				'style'   => array(),
				'title'   => array(),
				'data-id' => array(),
			),
			'span'     => array(
				'class' => array(),
				'title' => array(),
			),
			'textarea' => array(
				'id'          => array(),
				'name'        => array(),
				'class'       => array(),
				'rows'        => array(),
				'cols'        => array(),
				'placeholder' => array(),
			),
			'img'      => array(
				'src' => array(),
			),
			'h2'       => array(),
			'button'   => array(),
			'p'        => array(),
			'i'        => array(),
			'br'       => array(),
		);
		print wp_kses( $output, $allowed_html );
	}

	/**
	 * Set envelope from address.
	 *
	 * @param PHPMailer $phpmailer PHPMailer instance.
	 */
	public function phpmailer_setfrom( $phpmailer ) {
		$subject_exp = explode( 'EQ3_From:', $phpmailer->Subject );
		if ( isset( $subject_exp[1] ) ) {
			$phpmailer->setFrom( $subject_exp[1] );
			$phpmailer->Subject = $subject_exp[0];
		}
	}

	/**
	 * Set the page title for the 'View Feedback' page.
	 *
	 * @param string $title existing page title.
	 * @param int $id page ID.
	 *
	 * @return string modified title.
	 */
	public function feedback_page_title( $title, $id = null ) {
		if ( ! in_the_loop() ) {
			// returns early if not relevant (as in custom menu loops).
			return $title;
		}
		if ( 'View Feedback' === $title ) {
			$options = eq3f_get_options_new();
			if ( ! empty( $options['feedback_page_labels_page_title'] ) ) {
				return $options['feedback_page_labels_page_title'];
			}
		} else {
			return $title;
		}
	}

	/**
	 * Add routes
	 */
	public function remsto_feedback_routes() {
		add_rewrite_tag( '%rf_action%', '([^&]+)' );
	}

  /**
	 * Handle request process
	 *
	 * @param type $wpq WP query.
	 */
	public function remsto_feedback_pages( $wpq ) {
		$action = isset( $wpq->query_vars['rf_action'] ) ? $wpq->query_vars['rf_action'] : '';
		switch ( $action ) {
			case 'document':
				print 'HELLO';
				exit;
		}
	}

  /**
   * Get labels for feedback form
   */
  private function remsto_client_feedback_get_form_labels($parent_feedback_settings) {
    $labels = array(
      'page_title' =>
        $parent_feedback_settings['feedback_page_title'],
      //'title' =>
      //  $parent_feedback_settings['feedback_title'],
      'intro_text' =>
        $parent_feedback_settings['feedback_intro_text'],
      'stood_out' =>
        $parent_feedback_settings['feedback_stood_out_label'],
      'recommend' =>
        $parent_feedback_settings['feedback_recommend_label'],
      'staff_service' =>
        $parent_feedback_settings['feedback_staff_service_label'],
      'rating_promised' =>
        $parent_feedback_settings['feedback_rating_promised_label'],
      'rating_value' =>
        $parent_feedback_settings['feedback_rating_value_label'],
      'rating_helpful' =>
        $parent_feedback_settings['feedback_rating_helpful_label'],
      'rating_communication' =>
        $parent_feedback_settings['feedback_rating_communication_label'],
      'better' =>
        $parent_feedback_settings['feedback_better_label'],
      'anonymous' =>
        $parent_feedback_settings['feedback_anonymous_label'],
      'concealed' =>
        $parent_feedback_settings['feedback_concealed_label'],
      'your_details_text' =>
        $parent_feedback_settings['feedback_your_details_text'],
      'first_name' =>
        $parent_feedback_settings['feedback_first_name_label'],
      'last_name' =>
        $parent_feedback_settings['feedback_last_name_label'],
      'email' =>
        $parent_feedback_settings['feedback_email_label'],
      'optional_text' =>
        $parent_feedback_settings['feedback_optional_text'],
      'town' =>
        $parent_feedback_settings['feedback_town_label'],
      'county' =>
        $parent_feedback_settings['feedback_county_label'],
      'telephone' => 
        $parent_feedback_settings['feedback_telephone_label'],
      'company_name' =>
        $parent_feedback_settings['feedback_company_label'],
      'business_category' =>
        $parent_feedback_settings['feedback_business_category_label'],
      'position' => 
        $parent_feedback_settings['feedback_position_label'],
      'website' =>
        $parent_feedback_settings['feedback_website_label'],
      //'services_provided' =>
      //  $parent_feedback_settings['feedback_services_provided_label'],
      'extra_rating_questions' =>
        $parent_feedback_settings['feedback_extra_rating_questions_label'],
      'extra_worded_questions' =>
        $parent_feedback_settings['feedback_extra_worded_questions_label'],
    );
    return $labels;
  }

  private function remsto_client_feedback_get_form_weights($additional_vocabularies, $parent_feedback_settings) {
    $weights = array(
      'intro_text' =>
        $parent_feedback_settings['feedback_intro_text_weight'],
      'stood_out' =>
        $parent_feedback_settings['feedback_stood_out_weight'],
      'recommend' =>
        $parent_feedback_settings['feedback_recommend_weight'],
      'staff_service' =>
        $parent_feedback_settings['feedback_staff_service_weight'],
      'rating_promised' =>
        $parent_feedback_settings['feedback_rating_promised_weight'],
      'rating_value' =>
        $parent_feedback_settings['feedback_rating_value_weight'],
      'rating_helpful' =>
        $parent_feedback_settings['feedback_rating_helpful_weight'],
      'rating_communication' =>
        $parent_feedback_settings['feedback_rating_communication_weight'],
      'better' =>
        $parent_feedback_settings['feedback_better_weight'],
      'your_details_group' =>
        $parent_feedback_settings['feedback_your_details_group_weight'],
      'optional_group' =>
        $parent_feedback_settings['feedback_optional_group_weight'],
      'services' =>
        $parent_feedback_settings['feedback_services_weight'],
      'more_services_group' =>
        $parent_feedback_settings['feedback_more_services_group_weight'],
      'extra_rating_questions' =>
        $parent_feedback_settings['feedback_extra_rating_questions_weight'],
      'extra_worded_questions' =>
        $parent_feedback_settings['feedback_extra_worded_questions_weight'],
    );

    if(is_array($weights['more_services_group']) && count($weights['more_services_group'])==0) {
      $weight=$default_weights['more_services_group'];
      foreach($additional_vocabularies as $additional_vocabulary) {
        if($additional_vocabulary>0) {
          $weights['more_services_group'][$additional_vocabulary]=$weight;
          $weight++;
        }
      }
    }
    if(is_array($weights['extra_rating_questions']) && count($weights['extra_rating_questions'])==0) {
      $weight=$default_weights['extra_rating_questions'];
      for($i=0; $i<4; $i++) {
        $weights['extra_rating_questions'][$i]=$weight;
        $weight++;
      }
    }
    if(is_array($weights['extra_worded_questions']) && count($weights['extra_worded_questions'])==0) {
      $weight=$default_weights['extra_worded_questions'];
      for($i=0; $i<4; $i++) {
        $weights['extra_worded_questions'][$i]=$weight;
        $weight++;
      }
    }
    return $weights;
  }

  /**
   * CSV Header for summary report.
   * @return string
   */
  private function remsto_client_feedback_admin_csv_export_summary_header() {

    $header = array(
      'create_date' => 'Time',
      'stood_out' => 'What stood out?',
      'recommend' => 'Refer?',
      'staff_service' => 'Staff/Service',
      'rating_promised' => 'Delivery',
      'rating_value' => 'Value',
      'rating_helpful' => 'Helpful',
      'rating_communication' => 'Comms',
      'better' => 'Improvement Suggestion?',
      'anonymous' => 'Anon?',
      'concealed' => 'Concealed?',
      'first_name' => 'First',
      'last_name' => 'Last',
      'email' => 'Email',
      'company' => 'Company',
    );

    return $header;
  }

  /**
   * Header for detail report.
   * @return string
   */
  function remsto_client_feedback_admin_csv_export_detail_header($labels, $weights, $vocabularies, $taxonomy_details, $display_optional) {
    $row=array();
    $row[-1000]='Time';
    $row[$weights['stood_out']]=$labels['stood_out'];
    $row[$weights['recommend']]=$labels['recommend'];
    if(isset($labels['staff_service']) && strlen($labels['staff_service'])>0) {
      $row[$weights['staff_service']]=$labels['staff_service'];
    }
    $row[$weights['rating_promised']]=$labels['rating_promised'];
    $row[$weights['rating_value']]=$labels['rating_value'];
    $row[$weights['rating_helpful']]=$labels['rating_helpful'];
    $row[$weights['rating_communication']]=$labels['rating_communication'];
    if(isset($labels['better']) && strlen($labels['better'])>0) {
      $row[$weights['better']]=$labels['better'];
    }
    $row[($weights['your_details_group']+1)]='Anonymous';
    $row[($weights['your_details_group']+2)]='Concealed';
    $row[($weights['your_details_group']+3)]='First name';
    $row[($weights['your_details_group']+4)]='Last name';
    $row[($weights['your_details_group']+5)]='Email';
    if($display_optional && isset($labels['optional_text']) && strlen($labels['optional_text'])>0) {
      $row[($weights['optional_group']+1)]='Town';
      $row[($weights['optional_group']+2)]='County';
      $row[($weights['optional_group']+3)]='Telephone';
      $row[($weights['optional_group']+4)]='Company name';
      $row[($weights['optional_group']+5)]='Business category';
      $row[($weights['optional_group']+6)]='Position';
      $row[($weights['optional_group']+7)]='Website';
    }
    foreach ($vocabularies as $tid) {
      if ($tid > 0) {
        $row[$weights['more_services_group'][$tid]] = $taxonomy_details[$tid]['title'];
      }
    }
    for($i=0; $i<4; $i++) {
      if(isset($labels['extra_rating_questions'][$i]) && strlen($labels['extra_rating_questions'][$i])>0) {
        $column = 'extra_rating_question_'.($i+1);
        $row[$weights['extra_rating_questions'][$i]] = $labels['extra_rating_questions'][$i];
      }
    }
    for($i=0; $i<4; $i++) {
      if(isset($labels['extra_worded_questions'][$i]) && strlen($labels['extra_worded_questions'][$i])>0) {
        $column = 'extra_worded_question_'.($i+1);
        $row[$weights['extra_worded_questions'][$i]] =$labels['extra_worded_questions'][$i];
      }
    }
    $row[] = 'Company comment';
    $row[] = 'Video testimonial';
    ksort($row);

    return $row;
  }

  /**
   * Summary CSV record
   * @param object $fb Feedback record
   * @return type
   */
  private function remsto_client_feedback_admin_csv_export_summary_row($fb) {

    $row = array(
      'create_date' => date('d/m/Y H:i', $fb->created),
      'stood_out' => $fb->stood_out,
      'recommend' => $this->remsto_client_feedback_admin_yes_no_text($fb->recommend),
      'staff_service' => $fb->staff_service,
      'rating_promised' => $this->remsto_client_feedback_percentage_to_value($fb->rating_promised),
      'rating_value' => $this->remsto_client_feedback_percentage_to_value($fb->rating_value),
      'rating_helpful' => $this->remsto_client_feedback_percentage_to_value($fb->rating_helpful),
      'rating_communication' => $this->remsto_client_feedback_percentage_to_value($fb->rating_communication),
      'better' => $fb->better,
      'anonymous' => $this->remsto_client_feedback_admin_yes_no_text($fb->anonymous),
      'concealed' => $this->remsto_client_feedback_admin_yes_no_text($fb->concealed),
      'first_name' => $fb->first_name,
      'last_name' => $fb->last_name,
      'email' => $fb->email,
      'company' => $fb->company_name,
    );

    return $row;
  }

  /**
   * Detail CSV record
   * @param object $fb Feedback record
   * @return type
   */
  private function remsto_client_feedback_admin_csv_export_detail_row($fb, $labels, $weights, $vocabularies, $taxonomy_details, $display_optional) {
    $row=array();
    $row[-1000]=date('d/m/Y H:i', $fb->created);
    $row[$weights['stood_out']]=$fb->stood_out;
    $row[$weights['recommend']]=$fb->recommend<0?'N/A':($fb->recommend>0?'Yes':'No');
    if(isset($labels['staff_service']) && strlen($labels['staff_service'])>0) {
      $row[$weights['staff_service']]=$fb->staff_service;
    }
    $row[$weights['rating_promised']]=$this->remsto_client_feedback_percentage_to_value($fb->rating_promised);
    $row[$weights['rating_value']]=$this->remsto_client_feedback_percentage_to_value($fb->rating_value);
    $row[$weights['rating_helpful']]=$this->remsto_client_feedback_percentage_to_value($fb->rating_helpful);
    $row[$weights['rating_communication']]=$this->remsto_client_feedback_percentage_to_value($fb->rating_communication);
    if(isset($labels['better']) && strlen($labels['better'])>0) {
      $row[$weights['better']]=$fb->better;
    }
    //if(isset($labels['your_details_group']) && strlen($labels['your_details_group'])>0) {
      $row[($weights['your_details_group']+1)]=$fb->anonymous>0?'Yes':'No';
      $row[($weights['your_details_group']+2)]=$fb->concealed>0?'Yes':'No';
      $row[($weights['your_details_group']+3)]=$fb->first_name;
      $row[($weights['your_details_group']+4)]=$fb->last_name;
      $row[($weights['your_details_group']+5)]=$fb->email;
    //}
    if($display_optional && isset($labels['optional_text']) && strlen($labels['optional_text'])>0) {
      $row[($weights['optional_group']+1)]=$fb->town;
      $row[($weights['optional_group']+2)]=$fb->county;
      $row[($weights['optional_group']+3)]=$fb->telephone;
      $row[($weights['optional_group']+4)]=$fb->company_name;
      $row[($weights['optional_group']+5)]=$fb->business_category;
      $row[($weights['optional_group']+6)]=$fb->position;
      $row[($weights['optional_group']+7)]=$fb->website;
    }
    $tax = $fb->taxonomy_all;
    foreach ($vocabularies as $tid) {
      if ($tid > 0) {
        if (isset($tax->$tid)) {
          $row[$weights['more_services_group'][$tid]] = $this->remsto_client_feedback_admin_format_services($tax->$tid, $taxonomy_details[$tid], ',');
        }
        else {
          $row[$weights['more_services_group'][$tid]] = '';
        }
      }
    }
    for($i=0; $i<4; $i++) {
      if(isset($labels['extra_rating_questions'][$i]) && strlen($labels['extra_rating_questions'][$i])>0) {
        $column = 'extra_rating_question_'.($i+1);
        $row[$weights['extra_rating_questions'][$i]] = $this->remsto_client_feedback_percentage_to_value($fb->$column);
      }
    }
    for($i=0; $i<4; $i++) {
      if(isset($labels['extra_worded_questions'][$i]) && strlen($labels['extra_worded_questions'][$i])>0) {
        $column = 'extra_worded_question_'.($i+1);
        $row[$weights['extra_worded_questions'][$i]] = $fb->$column;
      }
    }
    $row[] = $fb->company_comment;
    $row[] = $fb->video_testimonial;
    ksort($row);
    return $row;
  }

  /**
   * Convert percentage ratting into value
   * @param type $percentage
   * @return number
   */
  private function remsto_client_feedback_percentage_to_value($percentage) {

    if ($percentage > 0) {
      $value = 5 * ($percentage / 100);
    }
    else {
      $value = "N/A";
    }

    return $value;
  }

  private function remsto_client_feedback_admin_yes_no_text($value) {

    if ($value === '1') {
      $value_text = 'Yes';
    }
    else {
      $value_text = 'No';
    }

    return $value_text;
  }

  /**
   * Show taxonomy as string
   * @param array $tax taxonomy selections
   * @param array $term_details details of taxonomy terms
   * @param string $seperator separator used to separate terms (default = space)
   * @return type
   */
  private function remsto_client_feedback_admin_format_services($tax, $term_details, $seperator = ' ') {

    $txt = '';

    foreach ($tax as $term_id) {
      $term = $term_details[$term_id];
      $txt .= $term['title'] . $seperator;
    }

    // Remove extra spacer
    return rtrim($txt, $seperator);
  }

}

// End of class.

register_activation_hook( __FILE__, array( 'EQ3F_Feedback', 'activate' ) );
register_deactivation_hook( __FILE__, array( 'EQ3F_Feedback', 'deactivate' ) );
