<?php
/**
 * Enquir3 feedback management table.
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * Class: EQ3F_Feedback_Management_Table
 */
class EQ3F_Feedback_Management_Table extends WP_List_Table {

	/**
	 * Instance of feedback client.
	 *
	 * @var EQ3F_Feedback $eq3f_feedback
	 */
	protected $eq3f_feedback;

	/**************************************************************************
	 * REQUIRED. Set up a constructor that references the parent constructor. We
	 * use the parent reference to set some default configs.
	 * ************************************************************************* */
	/**
	 * Constructor
	 *
	 * @param type $eq3f_feedback Feedback plugin instance.
	 */
	public function __construct( $eq3f_feedback ) {
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_style( 'eq3f-rating-widget' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );

		wp_localize_script(
			'eq3f-feedback-ajax-client-script',
			'remsto_ajax_settings',
			array( 'ajax_url' => admin_url( 'admin-ajax.php' ) )
		);

		// Set parent defaults.
		parent::__construct(
			array(
				'singular' => 'feedback', // singular name of the listed records.
				'plural'   => 'feedback', // plural name of the listed records.
				'ajax'     => false, // does this table support ajax?
			)
		);
		$this->eq3f_feedback = $eq3f_feedback;
	}

	/**
	 * Defaults for columns
	 *
	 * @param array $item All columns.
	 * @param str   $column_name Name of column.
	 *
	 * @return string Data to display
	 */
	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'id':
			case 'title':
			case 'status':
			case 'created':
			case 'first_name':
			case 'last_name':
			case 'company':
			case 'refer':
			case 'delivery':
			case 'value':
			case 'helpful':
			case 'comms':
			case 'view':
				return $item[ $column_name ];
			default:
				return 'Error occurred';
		}
	}

	/**
	 * Add row actions
	 *
	 * @param array $item Columns array.
	 *
	 * @return type
	 */
	public function column_title( $item ) {

		$page = filter_input( INPUT_GET, 'page' );

		// Build row actions.
		$actions = array(
			//'edit'   => '<a href="?page=' . $page . '&action=edit&fid=' . $item['id'] . '">Edit</a>',
			'edit'   => '<a href="?page=' . $page . '&action=edit&fid=' . $item['id'] . '">Edit</a>',
			'delete' => '<a href="?page=' . $page . '&action=delete&fid=' . $item['id'] . '">Delete</a>',
		);

		// Add link for casestudy if plugin enabled and no casestudy exists for this feedback item.
		if ( is_plugin_active( 'enquir3-case-studies/enquir3-case-studies.php' ) ) {
			// See if a case study exists.
			try {
				$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
				$casestudy_client = new RemstoCaseStudyClient( $feedback_options );
				$cs               = $casestudy_client->getByFeedbackId( $item['id'] );
			} catch ( Exception $exc ) {
				$cs = null;
			}

			if ( $cs->id ) {
				$edit_link            = admin_url( 'admin.php?page=casestudy-management&action=edit&cid=' . $cs->id );
				$actions['casestudy'] = "<a href='$edit_link'>Edit Case Study</a>";
			} else {
				$edit_link            = admin_url( 'admin.php?page=casestudy-management&action=edit&fid=' . $item['id'] );
				$actions['casestudy'] = "<a href='$edit_link'>Create Case Study</a>";
			}
		}

		// Return the title contents + actions.
		return $item['title'] . $this->row_actions( $actions );
	}


	/**
	 * Checkbox columns
	 *
	 * @param array $item Columns.
	 *
	 * @return str
	 */
	public function column_cb( $item ) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			$this->_args['singular'],
			$item['id']
		);
	}

	/**
	 * Get columns
	 *
	 * @return str
	 */
	public function get_columns() {
		$columns = array(
			'cb'         => '<input type="checkbox" />',
			'id'         => '#',
			'created'    => 'Created',
			'status'     => 'Status',
			'title'      => 'Stood Out',
			'first_name' => 'First Name',
			'last_name'  => 'Last Name',
			'company'    => 'Company',
			'refer'      => 'Refer',
			'delivery'   => 'Delivery',
			'value'      => 'Value',
			'helpful'    => 'Helpful',
			'comms'      => 'Comms',
		);
		return $columns;
	}

	/**
	 * Prepare rows
	 */
	public function prepare_items() {
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClientSiteStatus.inc';

    $this->process_bulk_action();

		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_style( 'eq3f-feedback-admin' );

		// Get current page number for this page.
		$page_number = $this->get_pagenum() - 1;

		// Column headings.
		$columns               = $this->get_columns();
		$hidden                = array();
		$sortable              = array();
		$this->_column_headers = array( $columns, $hidden, $sortable );

		// Get the feedback data.
		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		if ( isset( $feedback_options['service_taxonomy'] ) ) {
			$service_taxonomy = $feedback_options['service_taxonomy'];
		} else {
			$service_taxonomy = null;
		}

		$filter = array(
			'site_id'     => $feedback_options['client_id'],
			'parent_type' => 0,
			'parent_id'   => 0,
			'taxonomy_id' => $service_taxonomy,
			'deleted'     => 0,
		);

		$filter_published = filter_input( INPUT_GET, 'filter-published' );
		if ( strlen( $filter_published ) > 0 && 'all' !== $filter_published ) {
			$filter['site_status_eq'] = $filter_published;
		}

		$filter_items_per_page = filter_input( INPUT_GET, 'filter-items-per-page' );
		if ( strlen( $filter_items_per_page ) > 0 ) {
			$items_per_page = $filter_items_per_page;
		}
    else {
      $items_per_page = 10;
    }

		$sort = array(
			'order' => 'created',
			'sort'  => 'DESC',
		);

		$feedback_data = $this->eq3f_feedback->get_feedback_client()->listFeedback( 0, 0, $filter, $sort, $items_per_page, $page_number );

		$rows = array();
		foreach ( $feedback_data->data as $feedback ) {

			$stood_out = sanitize_text_field( RemstoModuleFunctions::truncate_utf8( $feedback->stood_out, 120, true, true ) );

			switch ( $feedback->recommend ) {
				case 0:
					$refer = 'No';
					break;
				case 1:
					$refer = 'Yes';
					break;
				default:
					$refer = 'N/A';
			}
			$rows[] = array(
				'id'         => $feedback->id,
				'title'      => $stood_out,
				'first_name' => $feedback->first_name,
				'last_name'  => $feedback->last_name,
				'company'    => $feedback->company_name,
				'refer'      => $refer,
				'status'     => RemstoClientSiteStatus::toText( $feedback->status ),
				'created'    => gmdate( 'd-m-Y H:i:s', $feedback->created ),
				'delivery'   => $this->displayRating( $feedback->rating_promised ),
				'value'      => $this->displayRating( $feedback->rating_value ),
				'helpful'    => $this->displayRating( $feedback->rating_helpful ),
				'comms'      => $this->displayRating( $feedback->rating_communication ),
				'view'       => RemstoModuleFunctions::feedback_more_button(
					array(
						'id'    => $feedback->id,
						'label' => 'view',
						'title' => '',
					)
				),
			);
		}
		/**
		 * REQUIRED. Now we can add our *sorted* data to the items property, where
		 * it can be used by the rest of the class.
		 */
		$this->items = $rows;

		// Set up the pager.
		$this->set_pagination_args(
			array(
				'total_items' => $feedback_data->total,
				'per_page'    => $items_per_page,
				'total_pages' => ceil( $feedback_data->total / $items_per_page ),
			)
		);
	}

	/**
	 * Convert percentage score to number
	 *
	 * @param type $score Percentage.
	 * @return int
	 */
	public function displayRating( $score ) {
		if ( $score > 0 ) {
			return $score / 20;
		} else {
			return 0;
		}
	}

	/**
	 * Extra table navigation
	 *
	 * @param type $which Top.
	 */
	public function extra_tablenav( $which ) {
		if ( 'top' === $which ) {
      $published_options = array(
        0 => 'Unpublished',
        1 => 'Published',
        2 => 'Promoted',
      );
			// Add a filter for 'published'.
			$filter_published = filter_input( INPUT_GET, 'filter-published' );
			print '<div class="alignleft actions bulkactions">';
			print '<select name="filter-published" class="filter-published">';
			print '<option value="">-- Filter by Published --</option>';
			foreach ( $published_options as $option_id => $option_title ) {
				$selected = '';
				if ( isset( $filter_published ) && '' !== $filter_published && intval( $filter_published ) === $option_id ) {
					$selected = ' selected = "selected"';
				}
				print '<option value="' . esc_attr( $option_id ) . '"' . esc_attr( $selected ) . '>' . esc_attr( $option_title ) . '</option>';
			}
			print '</select>';

      $items_per_page_options = array(
        5  => '5 items per page',
        10 => '10 items per page',
        20 => '20 items per page',
        50 => '50 items per page',
      );

      // Add a filter for 'items per page'.
			$filter_items_per_page = filter_input( INPUT_GET, 'filter-items-per-page' );
      if ( !isset($filter_items_per_page) || '' === $filter_items_per_page ) {
        $filter_items_per_page = '10';
      }
			print '<div class="alignleft actions bulkactions">';
			print '<select name="filter-items-per-page" class="filter-items-per-page">';
			foreach ( $items_per_page_options as $option_id => $option_title ) {
				if ( intval( $filter_items_per_page ) === $option_id ) {
					$selected = ' selected = "selected"';
				} else {
          $selected = '';
        }
				print '<option value="' . esc_attr( $option_id ) . '"' . esc_attr( $selected ) . '>' . esc_attr( $option_title ) . '</option>';
			}
			print '</select>';

			submit_button( 'Filter', 'secondary', false, false );
			print '</div>';
		}
	}

  /**
   * Returns an associative array containing the bulk action
   *
   * @return array
   */
  public function get_bulk_actions() {
    $actions = [
      'bulk-delete' => 'Delete'
    ];

    return $actions;
  }

  public function process_bulk_action() {

    // If the delete bulk action is triggered...
    if ('bulk-delete' === $this->current_action()) {
      $get = filter_input_array( INPUT_GET );

      $delete_ids = $get['feedback'];

      $this->eq3f_feedback->remsto_feedback_management_render_page( 'delete', $delete_ids );
    }
  }

}
