<?php
/**
 * Taxonomy vocabulary list.
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * Display list of taxonomy vocabularies.
 */
class EQ3F_Feedback_Taxonomy_List extends WP_List_Table {

	/**
	 * Instance of feedback client.
	 *
	 * @var RemstoFeedbackClient $feedback_client
	 */
	protected $feedback_client;

	/**
	 * Constructor
	 *
	 * @param type $feedback_client Feedback client.
	 */
	public function __construct( $feedback_client ) {

		// Set parent defaults.
		parent::__construct(
			array(
				'singular' => 'taxonomy', // singular name of the listed records.
				'plural'   => 'taxonomy', // plural name of the listed records.
				'ajax'     => false,        // does this table support ajax?
			)
		);
		$this->feedback_client = $feedback_client;
	}

	/**
	 * Defaults for columns
	 *
	 * @param array $item All columns.
	 * @param str   $column_name Name of column.
	 *
	 * @return string Data to display
	 */
	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'name':
				return $this->add_actions( $item[ $column_name ], $item['tid'] );
			case 'type':
				return $item[ $column_name ];
			default:
				return 'Something went wrong';
		}
	}

	/**
	 * Add actions to cell
	 *
	 * @param str $cell_data Cell data.
	 * @param str $tid Taxonomy vocabulary ID.
	 *
	 * @return string Data to display
	 */
	public function add_actions( $cell_data, $tid ) {
		$page = filter_input( INPUT_GET, 'page' );
		// Build row actions.
		$actions = array(
			'edit'  => "<a href='?page=$page&action=edit&tid=$tid'>Edit</a>",
			'terms' => "<a href='?page=$page&action=terms&tid=$tid'>Terms</a>",
		);

		// Return the title contents.
		return $cell_data . $this->row_actions( $actions );
	}

	/**
	 * Get columns
	 *
	 * @return str
	 */
	public function get_columns() {
		$columns = array(
			'name' => 'Taxonomy Name',
			'type' => 'Type',
		);
		return $columns;
	}

	/**
	 * Prepare rows
	 */
	public function prepare_items() {
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClientSiteStatus.inc';

		// Column headings.
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = array();

		$this->_column_headers = array( $columns, $hidden, $sortable );

		$taxonomies = $this->feedback_client->listVocabularies();

		$rows = array();
		foreach ( $taxonomies as $taxonomy ) {

			// Only show global taxonomies that allow local additions.
			$allow_local_additions = RemstoClientTaxonomyDisplayOptions::isOptionSet( $taxonomy->display_options, RemstoClientTaxonomyDisplayOptions::ALLOW_LOCAL_ADDITIONS );
			if ( RemstoClientTaxonomyTypes::GLOBAL_TAXONOMY === $taxonomy->type && ! $allow_local_additions ) {
				continue;
			}

			if ( '1' === $taxonomy->type ) {
				$type_str = 'Local';
			} else {
				$type_str = 'Global';
			}

			$rows[] = array(
				'tid'  => $taxonomy->tid,
				'name' => $taxonomy->title,
				'type' => $type_str,
			);
		}

		$this->items = $rows;
	}

}
