<?php
/**
 * Taxonomy terms list.
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * Display list of taxonomy vocabularies.
 */
class EQ3F_Feedback_Taxonomy_Term_List extends WP_List_Table {

	/**
	 * Instance of feedback client.
	 *
	 * @var RemstoFeedbackClient $feedback_client
	 */
	protected $feedback_client;

	/**
	 * Instance of feedback client.
	 *
	 * @var int $tid Taxonomy vocabulary ID.
	 */
	protected $tid;

	/**
	 * Constructor
	 *
	 * @param type $feedback_client Feedback client.
	 * @param int  $tid Taxonomy vocabulary ID.
	 */
	public function __construct( $feedback_client, $tid = null ) {

		// Set parent defaults.
		parent::__construct(
			array(
				'singular' => 'taxonomy', // singular name of the listed records.
				'plural'   => 'taxonomy', // plural name of the listed records.
				'ajax'     => false,      // does this table support ajax?
			)
		);
		$this->feedback_client = $feedback_client;
		$this->tid             = $tid;
	}

	/**
	 * Defaults for columns
	 *
	 * @param array $item All columns.
	 * @param str   $column_name Name of column.
	 *
	 * @return string Data to display
	 */
	public function column_default( $item, $column_name ) {

		switch ( $column_name ) {
			case 'name':
				return $this->add_actions( $item[ $column_name ], $item['ttid'] );
			case 'type':
				return $item[ $column_name ];
			case 'position':
				return $item[ $column_name ];
			default:
				return 'Something went wrong';
		}
	}

	/**
	 * Add actions to cell
	 *
	 * @param str $cell_data Cell data.
	 * @param str $ttid term ID.
	 *
	 * @return string Data to display
	 */
	public function add_actions( $cell_data, $ttid ) {

		if ( '' !== $ttid ) {
			$page = filter_input( INPUT_GET, 'page' );
			// Build row actions.
			$actions = array(
				'edit'   => "<a href='?page=$page&action=term-edit&tid=$this->tid&ttid=$ttid'>Edit</a>",
				'delete' => "<a href='?page=$page&action=tdel&tid=$this->tid&ttid=$ttid'>Delete</a>",
			);
			$rtn     = $cell_data . $this->row_actions( $actions );
		} else {
			$rtn = $cell_data;
		}

		return $rtn;
	}

	/**
	 * Get column names
	 *
	 * @return str
	 */
	public function get_columns() {
		$columns = array(
			'name' => 'Term Name',
			'position' => 'Position',
			'type' => 'Type',
		);
		return $columns;
	}

	/**
	 * Prepare rows
	 */
	public function prepare_items() {
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoClientSiteStatus.inc';

		// Column headings.
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = array();

		$this->_column_headers = array( $columns, $hidden, $sortable );

		$taxonomy = $this->feedback_client->getTaxonomy( $this->tid );

		$rows = array();
		foreach ( $taxonomy->terms as $key => $term ) {

			if ( '0' === $term->type ) { // global.
				$ttid        = '';
				$delete_link = '';
				$type_str    = 'Global';
			} else {
				$ttid        = $term->term_id;
				$delete_link = 'delete link';
				$type_str    = 'Local';
			}

      $rows[] = array(
				'ttid' => $ttid,
				'name' => $term->title,
				'position' => (isset($term->position))?$term->position:'',
				'type' => $type_str,
			);
		}

		$this->items = $rows;
	}

}
