<?php

/**
 * 
 * Remote Feedback Backend API calls
 * 
 * Calls to backend
 * 
 * @category   Remsto
 * @package    Irun
 * @author     Ian Burnup
 * @copyright  Irun 2016
 */



if (class_exists('RemstoFeedbackClient')) {
  return;
}


class RemstoFeedbackClient extends RemstoClient {
  
  /**
   * Get feedback items
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @param int $item_status filter by item status
   * @param int $page_size page size for pager
   * @param int $page page to retrieve
   * @return array list of feedback items
   */
  public function getFeedback(
      $parent_type,
      $parent_id,
      $item_status = null,
      $page_size = 0,
      $page = 0, 
      $service = null, 
      $deleted = null) {

    try {

      $post_data = array(
        'filter' => array(
          'site_id' => $this->config['client_id'], // this is already set as default why include twice?
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
          'taxonomy_id' => $service,
          'deleted' => $deleted,
        ),
        'sort' => array(
          'order' => 'created',
          'sort' => 'DESC',
        ),
      );

      if ($page_size > 0) {
        $post_data['pager'] = array(
          'items_per_page' => $page_size,
          'page_number' => $page,
        );
      }

      if (isset($item_status)) {
        $post_data['filter']['site_status'] = $item_status;
      }

      if (isset($service)) {
        $post_data['filter']['service'] = $service;
      }
      
      $data = $this->sendData('/api/v1/feedback/get', $post_data);
      return $data->feedback;
    }
    catch (\Exception $ex) {
      $this->displayException($ex, 'RemstoFeedbackClient');
    }
  }
  
  /**
   * Get feedback between two dates.
   * @param type $parent_type
   * @param type $parent_id
   * @param type $from_date
   * @param type $to_date
   * @return type
   */
  public function getFeedbackExport($parent_type, $parent_id, $from_date, $to_date, $service = null) {

    try {
//variable_get('feedback_service_taxonomy', 0)
      $post_data = array(
        'filter' => array(
          'site_id' => $this->config['client_id'], // this is already set as default why include twice?
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
          'taxonomy_id' => $service,
          'date_range' => array('from' => $from_date, 'to' => $to_date),
        ),
        'sort' => array(
          'order' => 'created',
          'sort' => 'ASC',
        ),
      );

      $data = $this->sendData('/api/v1/feedback/get', $post_data);
      return $data->feedback;
    }
    catch (\Exception $ex) {
      $this->displayException($ex, 'RemstoFeedbackClient');
    }
  }

  /**
   * Get data for summary display
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @return array summary data
   */
  public function getFeedbackSummary($parent_type, $parent_id) {

    try {
      $post_data = array(
        'filter' => array(
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
        )
      );
      $data = $this->sendData('/api/v1/feedback/get/summary', $post_data);
      return $data->summary_data;
    }
    catch (\Exception $ex) {
      $this->displayException($ex ,'RemstoFeedbackClient');
    }
  }
  
  /**
   * Get data for performance display
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @return array performance data
   */
  public function getPerformance($parent_type, $parent_id) {

    try {
      $post_data = array(
        'filter' => array(
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
        )
      );
      $data = $this->sendData('/api/v1/feedback/get/performance', $post_data);
      return $data->performance_data;
    }
    catch (\Exception $ex) {
      $this->displayException($ex, 'RemstoFeedbackClient');
    }
  }
  
    /**
   * Get data for performance display
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @return array performance data
   */
  public function getRatings($parent_type, $parent_id) {

    try {
      $post_data = array(
        'filter' => array(
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
        )
      );
      $data = $this->sendData('/api/v1/feedback/get/ratings', $post_data);
      return $data->ratings_data;
    }
    catch (\Exception $ex) {
      $this->displayException($ex, 'RemstoFeedbackClient');
    }
  }
  
  /**
   * Get specified item of feedback
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @param int $item_id id of feedback item to get
   * @return object feedback data
   */
  public function getFeedbackItem($parent_type, $parent_id, $item_id) {

    try {
      if ($parent_type !== NULL) {
        $post_data = array(
          /* @todo use filter for consistancy later
            'filter' => array(
            'parent_type' => $parent_type,
            'parent_id' => $parent_id,
            ),
           * 
           */
          'parent' => array(
            'item_type' => 2,
            'item_id' => $item_id,
          )
        );
      }
      else {
        $post_data = array();
      }
      $data = $this->sendData("/api/v1/feedback/get/$item_id", $post_data);
      if (isset($data->site_status)) {
        $data->feedback->site_status = $data->site_status;
      }
      else {
        $data->feedback->site_status = array();
      }
      return $data->feedback;
    }
    catch (\Exception $ex) {
      $this->displayException($ex, 'RemstoFeedbackClient');
    }
  }

  /**
   * Get a random item of feedback.
   * @param int $parent_type parent type
   * @param int $parent_id parent id
   * @return object feedback item
   */
  public function getRandomFeedbackItem($parent_type, $parent_id, $service = NULL, $seq = NULL) {

    try {
      $post_data = array(
        'filter' => array(
          'parent_type' => $parent_type,
          'parent_id' => $parent_id,
          'seq' => $seq,
        )
      );

      if (isset($service)) {
        $post_data['filter']['service'] = $service;
      }

      $data = $this->sendData('/api/v1/feedback/random', $post_data);
      return $data->feedback;
    }
    catch (\Exception $ex) {
      $this->displayException($ex ,'RemstoFeedbackClient');
    }
  }

  /**
   * Add new feedback
   * @param array $feedback_form_data Feedback data
   * @return type
   */
  public function addFeedback(
    $feedback_form_data,
    $printable_file = null,
    $printable_name = null,
    $image_file = null,
    $image_name = null) {

    try {
      $post_data = array(
        'form_data' => $feedback_form_data,
        'printable_file' => $printable_file,
        'printable_name' => $printable_name,
        'image_file' => $image_file,
        'image_name' => $image_name,
      );
      $data = $this->sendData('/api/v1/feedback/put', $post_data);
      return $data->feedback_id->feedback_id;
    }
    catch (Exception $ex) {
      //watchdog_exception('RemstoFeedbackClient', $ex);
    }
  }

  /**
   * Update feedback
   * @param array $feedback_form_data Feedback data
   * @return type
   */
  public function updateFeedback(
    $feedback_form_data,
    $printable_file = null,
    $printable_name = null,
    $image_file = null,
    $image_name = null) {
    
    try {
      $post_data = array(
        'form_data' => $feedback_form_data,
        'printable_file' => $printable_file,
        'printable_name' => $printable_name,
        'image_file' => $image_file,
        'image_name' => $image_name,
      );
      $data = $this->sendData(
        '/api/v1/feedback/post/' . $feedback_form_data['feedback_id'],
        $post_data
      );
      return $data;
    }
    catch (Exception $ex) {
      //watchdog_exception('RemstoFeedbackClient', $ex);
    }
  }

  public function listFeedback($parent_type, $parent_id, $filter, $sort, $page_size = 0, $page = 0) {
    try {
      $post_data = array(
        'filter' => $filter,
        'sort' => $sort,
      );

      if ($page_size > 0) {
        $post_data['pager'] = array(
          'items_per_page' => $page_size,
          'page_number' => $page,
        );
      }
      $data = $this->sendData('/api/v1/feedback/get', $post_data);
      return $data->feedback;
    }
    catch (\Exception $ex) {
      $this->displayException($ex ,'RemstoFeedbackClient');
    }
  }

  public function deleteFeedbackItem($item_id) {
    try {
      $post_data = array();
      $data = $this->sendData("/api/v1/feedback/delete/$item_id", $post_data);
      if (!empty($data->delete_status)) {
        return true;
      }
    }
    catch (\Exception $ex) {
      $this->displayException($ex ,'RemstoFeedbackClient');
    }
    return false;
  }

  public function permanentlyDeleteFeedbackItem($item_id) {
    try {
      $post_data = array();
      $data = $this->sendData("/api/v1/feedback/permanently-delete/$item_id", $post_data);
      if (!empty($data->delete_status)) {
        return true;
      }
    }
    catch (\Exception $ex) {
      $this->displayException($ex ,'RemstoFeedbackClient');
    }
    return false;
  }

}


