<?php


if (class_exists('RemstoForm')) {
  return;
}

abstract class RemstoForm {
     
  
  abstract public function display();
  
  /**
   * Add text input to form.
   * @param type $settings
   * @return string
   */
  protected function addFormTextInput($settings) {
    
    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'size' => 60,
      'maxlength' => 120,
      'value' => '',
      'title' => '',
    ), $settings);
    
    extract($settings);
    
    if(!empty($title)) {
      $tool_tip = "<span class='remsto-form-tooltip-icon' title='$title'>&nbsp;</span>";
    }
    else {
      $tool_tip= '';
    }
    
    $input = '<div class="form-item form-type-textfield">';
    $input .= "<label for='edit-$name'>$label $tool_tip</label>";
    $input .= "<input id='edit-$name' name='$name' value='$value' "; 
    $input .= "size='$size' maxlength='$maxlength' class='form-text' type='text'></div>";
    
    return $input;
  }
  
  protected function addFormSelectInput($settings) {

    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'options' => array(),
      'default_value' => array(),
        ), $settings);

    extract($settings);

    $input = "<div class='form-item form-type-select form-type-select-$name'>";
    $input .= "<label for='edit-$name'>$label</label>";
    $input .= "<div><select id='edit-$name' name='$name'>  ";

    foreach ($options as $option_value => $option_label) {

      if ($option_value == $default_value) {
        $selected = 'selected="selected"';
      }
      else {
        $selected = '';
      }

      $input .= "<option value='$option_value' $selected>$option_label</option>";
    }

    $input .= '</select></div></div>';
    return $input;
  }

  /**
   * Add select element to form.
   * @param type $settings
   * @return type
   */
  protected function addFormCheckBoxGroupInput($settings) {
    
    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'options' => array(),
      'value' => array(),
    ), $settings);
    
    extract($settings);
    
    $input = '<div class="form-item form-type-checkboxes form-item-display-options">';
    $input .= "<label for='edit-$name'>$label</label><div>";
 
    
    foreach ($options as $option_value => $option_label) {

      if(in_array($option_value, $value)) {
        $checked = 'checked="checked"';
      }
      else {
        $checked = '';
      }
      
      $select_name = $name. '[' . $option_value . ']';
      
      $input .= "<br><input id='$name-$option_value' name='$select_name' value='$option_value' $checked type='checkbox'>"; 
      $input .= "<label for='$name-$option_value'>$option_label</label>";
      
    }
    
    $input .= '</div></div>';
    
    return $input;
  }
  
  /**
   * Single check box
   * @param type $settings
   * @return string
   */
  protected function addFormCheckBoxInput($settings) {

    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'value' => 0,
        ), $settings);

    extract($settings);

    $input = '<div class="form-item form-type-checkboxe">';
    $input .= "<label for='edit-$name'>$label</label><div>";

    if ($value == 1) {
      $checked = 'checked="checked"';
    }
    else {
      $checked = '';
    }

    $input .= "<input id='edit-$name' name='$name' value='$value' $checked type='checkbox'>";


    $input .= '</div></div>';

    return $input;
  }

  /**
   * Add form text area
   * @param type $settings
   * @return type
   */
  protected function addFormTextArea($settings) {
    
    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'rows' => 5,
      'cols' => 50,
      'value' => '',
      'tinymce' => true,
      'title' => '',
      'maxlength' => '',
    ), $settings);
    
    extract($settings);
    
    if($tinymce) {
      $class = 'textarea-tinymce';
    }
    else {
      $class = '';
    }
    
    if(!empty($title)) {
      $tool_tip = "<span class='remsto-form-tooltip-icon' title='$title'>&nbsp;</span>";
    }
    else {
      $tool_tip= '';
    }
    
    if(!empty($maxlength)) {
      $maxlength_markup = " maxlength='$maxlength' ";
    }
    else {
      $maxlength_markup = '';
    }   
          
    $input = '<div class="form-item form-type-textarea">';
    $input .= "<label for='edit-$name'>$label $tool_tip</label>";
    $input .= "<textarea id='edit-$name' name='$name' class='$class' rows='$rows' cols='$cols' $maxlength_markup placeholder='$title'>"; 
    $input .= "$value</textarea></div>";
    
    return $input;
   
}

  /**
   * Add form text area
   * @param type $settings
   * @return type
   * see https://codex.wordpress.org/Function_Reference/wp_editor for settings
   */
  protected function addWPFormTextArea($settings) {
    
    $settings = shortcode_atts(array(
      'name' => '',
      'value' => '',
      'label' => '',
      'additionl_settings' => array(),
    ), $settings);
    
    extract($settings);
    
    $additionl_settings['textarea_name'] = "edit-$name";
    
    $input = "<label for='edit-$name'>$label</label><div>";
    
    $content = $value;
    $editor_id = $name;
    $input .=  wp_editor($content, $editor_id, $additionl_settings);
    
    return $input;
  
  
}

  protected function addItemStatusSelect($site_status) {
    
    $checked = array(
      0 => '',
      1 => '',
      2 => '',
    );
    
    $checked[$site_status] = 'checked';

    $output = '<div class="form-item form-type-radios form-item-status">';
    $output .= '<label for="edit-item-site-status">Site Status'; 
    $output .= '<span class="form-required" title="This field is required.">*</span></label>';
    $output .= '<div id="edit-item-site-status" class="form-radios">';
    $output .= '<div class="form-item form-type-radio form-item-site-status">';
    $output .= '<input type="radio" id="edit-item-site-status-0" name="this_site_status"'; 
    $output .= ' value="0" ' . $checked[0]  . ' class="form-radio" />';  
    $output .= '<label class="option" for="edit-item-site-status-0">Unpublished</label></div>';
        
    $output .= '<div class="form-item form-type-radio form-item-site-status">';
    $output .= '<input type="radio" id="edit-item-site-status-1" name="this_site_status"';
    $output .= ' value="1" ' . $checked[1]  . ' class="form-radio" />';  
    $output .= '<label class="option" for="edit-item-site-status-1">Published</label></div>';
       
    $output .= '<div class="form-item form-type-radio form-item-site-status">';
    $output .= '<input type="radio" id="edit-item-site-status-2" name="this_site_status"';
    $output .= ' value="2" ' . $checked[2]  . ' class="form-radio" />';  
    $output .= '<label class="option" for="edit-item-site-status-2">Published and promoted</label>';
    $output .= '</div></div></div>';               

    return $output;
  }

    
/**
 * Add markup to form
 * @param type $markup
 * @return type
 */
    protected function addFormMarkup($markup) {
    
    return $markup;
   
}

  protected function addFileUpload($settings) {


    $settings = shortcode_atts(array(
      'name' => '',
      'label' => '',
      'description' => '',
      'additionl_settings' => array(),
        ), $settings);
    
    extract($settings);

    $output = "<div class='field-wrapper field-$name-image'>";
    $output .= "<label for='edit-$name'>$label</label>";
    $output .= "<input type='file' id='edit-$name' name='$name' size='60' class='form-file'>";
    $output .= "<div class='description'>$description</div>";
    $output .= '</div>';

    return $output;
  }

}

/**
 * Dummy class to allow calling element create methods.
 */
class formHelper extends RemstoForm {

  public function display() {
    
  }

  public function addFormSelectInput($settings) {
    return parent::addFormSelectInput($settings);
  }

  public function addFormTextInput($settings) {
    return parent::addFormTextInput($settings);
  }

  public function addFormCheckBoxInput($settings) {
    return parent::addFormCheckBoxInput($settings);
  }

}
