<?php

/**
 * 
 * Remote Listing Backend API calls
 * 
 * Calls to backend
 * 
 * @category   Remsto
 * @package    Irun
 * @author     Rob Bale
 * @copyright  Irun 2016
 */

if (class_exists('RemstoListingClient')) {
  return;
}

class RemstoListingClient extends RemstoClient {
  private static $listing_id_cache=array();
  private static $listing_alias_cache=array();
	
  public function generateNewListing($values = array()) {
    $listing = array(
      'parent_type' => RemstoClientItemTypes::NONE,
      'parent_id' => 0,
      'company_name' => '',
      'contact_first_name' => '',
      'contact_last_name' => '',
      'contact_position' => '',
      'telephone' => '',
      'email_address' => '',
      'website' => '',
      'location_name' => '',
      'location_street' => '',
      'location_additional' => '',
      'location_city' => '',
      'location_county' => '',
      'location_postcode' => '',
      'location_lat' => 0.0,
      'location_lon' => 0.0,
      'logo_src' => '',
      'logo_alt' => '',
      'number_of_clients' => 0,
      'number_of_staff' => 0,
      'business_to_private_ratio' => 0,
      'other_services' => '',
      'about_us' => '',
      'contact' => '',
      'twitter_url' => '',
      'facebook_url' => '',
      'linkedin_url' => '',
      'google_url' => '',
      'youtube_url' => '',
      'tumblr_url' => '',
      'meta_tags' => '',
      'meta_description' => '',
      'path_alias' => '',
      'created' => time(),
      'claimed' => 0,
      'client_id' => 0,
      'account_id' => 0,
      'feedback_settings' =>'',
    );

    // If default values are provided, pre-populate the listing.
    if (!empty($values)) {
      foreach ($values as $key => $value) {
        if (isset($listing[$key])) {
          $listing[$key] = $value;
        }
      }
    }

    return $listing;
  }

  public function getListing($listing_id, $services_vocabulary=null) {
    try {
      if(!isset(self::$listing_id_cache[$listing_id])) {
        $post_data=array(
            'services_tid'=>$services_vocabulary,
        );
        $data = $this->sendData(
                '/api/v1/listing/get/'.$listing_id,
                $post_data);
        self::$listing_id_cache[$listing_id]=array(
          'listing'=>$data->listing,
          'statistics'=>$data->statistics,
          'user_access'=>$data->user_access,
          'services'=>$data->services,
          'additional_services'=>(array)$data->additional_services,
        );
      }
      return self::$listing_id_cache[$listing_id];
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }
  
  public function getListingById($listing_id) {
    try {
      $post_data = array();
      $data = $this->sendData('/api/v1/listing/get/'.$listing_id, $post_data);
      return array(
        'listing'     => $data->listing,
        'statistics'  => $data->statistics,
        'user_access' => $data->user_access,
        'services'    => $data->services,
        'additional_services'=>(array)$data->additional_services,
      );
      #return TRUE;
	  
    } catch (Exception $ex) {
      //print ($ex->getMessage());
      throw $ex;
    }
  }
  
  /**
   * Delete specific listing
   * @param type $listing_id Listing ID
   * @throws Exception
   */
  public function permanentlyDeleteListing($listing_id) {
    try {
      $post_data = array();
      return $this->sendData('/api/v1/listing/permanently_delete/' . $listing_id, $post_data);
    }
    catch (Exception $ex) {
      throw $ex;
    }
  }

  public function getListingByAlias($alias, $services_vocabulary=null) {
    try {
      if(!isset(self::$listing_alias_cache[$alias])) {
        $post_data=array(
            'services_tid'=>$services_vocabulary,
        );
        $data = $this->sendData(
            '/api/v1/listing/get/alias/' . $alias,
             $post_data);
        self::$listing_alias_cache[$alias]=array(
          'listing'=>$data->listing,
          'statistics'=>$data->statistics,
          'user_access'=>$data->user_access,
          'services'=>$data->services,
          'additional_services'=>(array)$data->additional_services,
        );
      }
      return self::$listing_alias_cache[$alias];
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }

  public function getListingByGuid($guid) {
    try {
      $post_data=array();
      $data = $this->sendData(
          '/api/v1/listing/get/guid/' . $guid,
           $post_data);
      return array(
          'listing'=>$data->listing,
      );
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }

  public function updateListing($listing_id, $post_data) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/post/'.$listing_id,
              $post_data);
      return $data;
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
      throw $ex;
    }
  }

  public function createListing($post_data) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/put',
              $post_data);
      return $data;
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
      throw $ex;
    }
  }

  public function searchListings(
    $filter,
    $sort,
    $pager,
    $also_site_status=false,
    $also_taxonomy_terms=false) {
    try {
      $post_data=array(
        'filter'=>$filter,
        'sort'=>$sort,
        'pager'=>$pager,
        'also_site_status'=>$also_site_status,
        'also_taxonomy_terms'=>$also_taxonomy_terms,
      );
      $data = $this->sendData(
        '/api/v1/listing/get',
        $post_data);
      return $data->listings;
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }

  public function getListingSiteStatus($listing_id) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/site_status/get/'.$listing_id,
              array());
      return array(
          'site_status'=>$data->site_status,
      );
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }

  public function updateListingSiteStatus($listing_id, $post_data) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/site_status/post/'.$listing_id,
              $post_data);
      return $data;
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
      throw $ex;
    }
  }

  public function getListingUserAccess($listing_id) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/user_access/get/'.$listing_id,
              array());
      return array(
          'user_access_listing'=>$data->user_access_listing,
          'user_access_feedback'=>$data->user_access_feedback,
      );
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
    }
  }

  public function updateListingUserAccess($listing_id, $post_data) {
    try {
      $data = $this->sendData(
              '/api/v1/listing/user_access/post/'.$listing_id,
              $post_data);
      return $data;
    }
    catch (Exception $ex) {
      print ($ex->getMessage());
      throw $ex;
    }
  } 
}
