<?php
/**
 *
 *
 * Feedback mail template.
 *
 * @category   Remsto
 * @package    Irun
 * @author     Ian Burnup
 * @copyright  Irun 2016
 */

?>

<style>
	.labels {
		font-weight: bold;
	}
	.field {
		margin-bottom: 30px;
	}
	.field-yesno .label,
	.field-yesno .data,
	.field-rating .label,
	.field-rating .data,
	.field-short .label,
	.field-short .data {
		display: inline-block;
	}
	.field-yesno .label,
	.field-rating .label,
	.field-short .label {
		width: 30%;
	}
</style>

<span style="font-weight: bold">Feedback</span><br>

<?php
remsto_notification_render_feedback_elements($vars);

function remsto_notification_percentage_to_value( $percentage ) {
	if ( $percentage > 0 ) {
		$value = 5 * ($percentage / 100);
	} else {
		$value = "N/A";
	}
	return $value;
}

function remsto_notification_label_text($label, $text) {
	$output = '';
	$output .= '<span style="font-weight: bold">';
	$output .= $label;
	$output .= '</span>';
	$output .= '<br/>';
	$output .= $text;
	$output .= '<br/>';
	$output .= '<br/>';
	return $output;
}

/**
 * Detail CSV record
 * @param object $vars Feedback record
 * @return type
 */
function remsto_notification_render_feedback_elements($vars) {
	$row = array();
	
	$row[ -1000 ] = '<p>New feedback received at: '.date( 'd/m/Y H:i', $vars[ 'created' ] ).'</p>';

	$row[ $vars[ 'options' ][ 'feedback_stood_out_weight' ] ] = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_stood_out_label' ],
		$vars[ 'stood_out' ]
	);

	$row[ $vars[ 'options' ][ 'feedback_recommend_weight' ] ] = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_recommend_label' ],
		$vars[ 'recommend' ] < 0 ? 'N/A' : ($vars[ 'recommend' ] > 0 ? 'Yes' : 'No' )
	);

	if ( '0' === $vars['options']['feedback_staff_service_hidden'] &&
	isset( $vars[ 'options' ][ 'feedback_staff_service_label' ] ) &&
	strlen( $vars[ 'options' ][ 'feedback_staff_service_label' ] ) > 0 ) {
		$row[ $vars[ 'options' ][ 'feedback_staff_service_weight' ] ] = remsto_notification_label_text(
			$vars[ 'options' ][ 'feedback_staff_service_label' ],
			$vars[ 'staff_service' ]
		);
	}

	$row[ $vars[ 'options' ][ 'feedback_rating_promised_weight' ] ] = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_rating_promised_label' ],
		remsto_notification_percentage_to_value( $vars[ 'rating_promised' ] )
	);

	$row[ $vars[ 'options' ][ 'feedback_rating_value_weight' ] ] = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_rating_value_label' ],
		remsto_notification_percentage_to_value( $vars[ 'rating_value' ] )
	);

	$row[ $vars[ 'options' ][ 'feedback_rating_helpful_weight' ] ]  = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_rating_helpful_label' ],
		remsto_notification_percentage_to_value( $vars[ 'rating_helpful' ] )
	);

	$row[ $vars[ 'options' ][ 'feedback_rating_communication_weight' ] ] = remsto_notification_label_text(
		$vars[ 'options' ][ 'feedback_rating_communication_label' ],
		remsto_notification_percentage_to_value( $vars[ 'rating_communication' ] )
	);

	if ( '0' === $vars['options']['feedback_better_hidden'] &&
	isset( $vars[ 'options' ][ 'feedback_better_label' ] ) &&
	strlen( $vars[ 'options' ][ 'feedback_better_label' ] ) > 0 ) {
		$row[ $vars[ 'options' ][ 'feedback_better_weight' ] ] = remsto_notification_label_text(
			$vars[ 'options' ][ 'feedback_better_label' ],
			$vars[ 'better' ]
		);
	}
	$row[ ( $vars[ 'options' ][ 'feedback_your_details_group_weight' ] . '.1' ) ] = remsto_notification_label_text(
		$vars['options']['feedback_anonymous_label'],
		isset( $vars[ 'anonymous' ] ) ? ($vars[ 'anonymous' ] > 0 ? 'Yes' : 'No') : 'No'
	);
	$row[ ( $vars[ 'options' ][ 'feedback_your_details_group_weight' ] . '.2' ) ] = remsto_notification_label_text(
		$vars['options']['feedback_concealed_label'],
		isset( $vars[ 'concealed' ] ) ? ($vars[ 'concealed' ] > 0 ? 'Yes' : 'No') : 'No'
	);
	$row[ ( $vars[ 'options' ][ 'feedback_your_details_group_weight' ] . '.3' ) ] = remsto_notification_label_text(
		$vars['options']['feedback_first_name_label'],
		$vars[ 'first_name' ]
	);
	$row[ ( $vars[ 'options' ][ 'feedback_your_details_group_weight' ] . '.4' ) ] = remsto_notification_label_text(
		$vars['options']['feedback_last_name_label'],
		$vars[ 'last_name' ]
	);
	$row[ ( $vars[ 'options' ][ 'feedback_your_details_group_weight' ] . '.5' ) ] = remsto_notification_label_text(
		$vars['options']['feedback_email_label'],
		$vars[ 'email' ]
	);

	if ( '0' === $vars['options']['feedback_optional_group_hidden'] &&
	'on' === $vars['options']['feedback_display_optional_fields'] &&
	isset( $vars[ 'options' ][ 'feedback_optional_text' ] ) && strlen( $vars[ 'options' ][ 'feedback_optional_text' ] ) > 0 ) {
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.1' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_position_label'],
			$vars[ 'position' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.2' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_company_label'],
			$vars[ 'company_name' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.3' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_website_label'],
			$vars[ 'website' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.4' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_town_label'],
			$vars[ 'town' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.5' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_county_label'],
			$vars[ 'county' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.6' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_telephone_label'],
			$vars[ 'telephone' ]
		);
		$row[ ( $vars[ 'options' ][ 'feedback_optional_group_weight' ] . '.7' ) ] = remsto_notification_label_text(
			$vars['options']['feedback_business_category_label'],
			$vars[ 'business_category' ]
		);
	}


	if ( '0' === $vars['options']['feedback_services_hidden'] &&
	isset( $vars['service_taxonomy_all'] ) ) {
		$selected=array();
		if(isset( $vars[ 'taxonomy' ][ $vars[ 'service_taxonomy_all' ][ 'tid' ] ] )) {
			foreach($vars[ 'taxonomy' ][ $vars[ 'service_taxonomy_all' ][ 'tid' ] ] as $term_id) {
				$selected[]=$vars['service_taxonomy_all']['terms'][$term_id]['title'];
			}
		}
		$row[ ($vars[ 'options' ][ 'feedback_services_weight' ]) ] = remsto_notification_label_text(
			$vars['service_taxonomy_all']['title'],
			implode(', ', $selected)
		);
	}

	if ( isset( $vars[ 'feedback_additional_taxonomy_all' ] ) ) {
		foreach ( $vars[ 'feedback_additional_taxonomy_all' ] as $key => $vocabulary ) {
			if ( '0' === $vars[ 'options' ][ 'feedback_more_services_group_hidden' ][ $vocabulary[ 'tid' ] ] ) {
				$selected=array();
				if ( isset( $vars[ 'taxonomy' ][ $vocabulary[ 'tid' ] ] ) ) {
					foreach ( $vocabulary[ 'terms' ] as $term ) {
						if ( in_array( $term[ 'term_id' ], $vars[ 'taxonomy' ][ $vocabulary[ 'tid' ] ] ) ) {
							$selected[] = $term[ 'title' ];
						}
					}
				}
				$row[ ($vars[ 'options' ][ 'feedback_more_services_group_weight' ][$vocabulary['tid']]) ] = remsto_notification_label_text(
					$vocabulary['title'],
					implode(', ', $selected)
				);
			}
		}
	}

	for ( $i = 0; $i < 4; $i++ ) {
		if ( '0' === $vars['options']['feedback_extra_rating_questions_hidden'][$i] &&
		isset( $vars[ 'options' ][ 'feedback_extra_rating_questions_label' ][ $i ] ) &&
		strlen( $vars[ 'options' ][ 'feedback_extra_rating_questions_label' ][ $i ] ) > 0 ) {
			$column = 'extra_rating_question_' . ($i + 1);
			$row[ $vars[ 'options' ][ 'feedback_extra_rating_questions_weight' ][ $i ] ] = remsto_notification_label_text(
				$vars[ 'options' ][ 'feedback_extra_rating_questions_label' ][ $i ],
				remsto_notification_percentage_to_value( $vars[ $column ] )
			);
		}
	}

	for ( $i = 0; $i < 4; $i++ ) {
		if ( '0' === $vars['options']['feedback_extra_worded_questions_hidden'][$i] &&
		isset( $vars[ 'options' ][ 'feedback_extra_worded_questions_label' ][ $i ] ) &&
		strlen( $vars[ 'options' ][ 'feedback_extra_worded_questions_label' ][ $i ] ) > 0 ) {
			$column = 'extra_worded_question_' . ($i + 1);
			$row[ $vars[ 'options' ][ 'feedback_extra_worded_questions_weight' ][ $i ] ] = remsto_notification_label_text(
				$vars[ 'options' ][ 'feedback_extra_worded_questions_label' ][ $i ],
				$vars[ $column ]
			);
		}
	}

	/*$row[] = remsto_notification_label_text(
		'Company Comment',
		$vars[ 'company_comment' ]
	);
	$row[] = remsto_notification_label_text(
		'Video Testimonial',
		$vars[ 'video_testimonial' ]
	);*/

	ksort( $row, SORT_NUMERIC  );
	foreach($row as $element) {
		print $element;
	}
}
