<?php
/**
 * Latest feedback widget
 */

/**
 * Latest feedback widget
 */
class Eq3f_Latest_Feedback_Widget extends WP_Widget {

	/**
	 * Constructor
	 */
	public function __construct() {

		parent::__construct(
			'eq3f_latest_feedback_widget',
			'Latest Feedback',
			array(
				'description' =>
							'Display latest feedback',
			)
		);
	}

	/**
	 * Code to render options form
	 *
	 * @param type $instance Options.
	 */
	public function form( $instance ) {

		$widget_title = ( ! empty( $instance['widget_title'] ) ?
		esc_attr( $instance['widget_title'] ) :
		'' );

		$orientation = ( ! empty( $instance['orientation'] ) ?
		esc_attr( $instance['orientation'] ) :
		'landscape' );

		$feedback_status = ( ! empty( $instance['feedback_status'] ) ?
		esc_attr( $instance['feedback_status'] ) :
		'1' );

		$motion = ( ! empty( $instance['motion'] ) ?
		esc_attr( $instance['motion'] ) :
		'static' );

		$max_char = ( ! empty( $instance['max-char'] ) ?
		esc_attr( $instance['max-char'] ) :
		'150' );

		$items_to_show = ( ! empty( $instance['items-to-show'] ) ?
		esc_attr( $instance['items-to-show'] ) :
		'3' );

		$scroll_speed = ( ! empty( $instance['scroll-speed'] ) ?
		esc_attr( $instance['scroll-speed'] ) :
		'3000' );
		?>

	<p>
		<label for="
		<?php
		echo esc_attr( $this->get_field_id( 'widget_title' ) );
		?>
	">
		<?php echo esc_attr( 'Widget Title:' ); ?>			
			<input type="text" 
				id="<?php echo esc_attr( $this->get_field_id( 'widget_title' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'widget_title' ) ); ?>"
				value="<?php echo esc_attr( $widget_title ); ?>" />			
		</label>
	</p> 

	<p><label for="<?php echo esc_attr( $this->get_field_id( 'feedback_status' ) ); ?>">
		<?php echo 'Feedback Status:'; ?>		
			Published <input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'feedback_status' ) ); ?>" 
							id="<?php echo esc_attr( $this->get_field_id( 'feedback_status' ) ); ?>" <?php checked( $feedback_status, '1' ); ?> value="1">
			Promoted <input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'feedback_status' ) ); ?>" 
						id="<?php echo esc_attr( $this->get_field_id( 'feedback_status' ) ); ?>" <?php checked( $feedback_status, '2' ); ?> value="2">
		</label>
	</p> 

	<p><label for="<?php echo esc_attr( $this->get_field_id( 'orientation' ) ); ?>">
			<?php echo 'Orientation:'; ?>		
			Portrait <input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'orientation' ) ); ?>" 
							id="<?php echo esc_attr( $this->get_field_id( 'orientation' ) ); ?>" <?php checked( $orientation, 'portrait' ); ?> value="portrait">
			Landscape <input type="radio" name="<?php echo esc_attr( $this->get_field_name( 'orientation' ) ); ?>" 
							id="<?php echo esc_attr( $this->get_field_id( 'orientation' ) ); ?>" <?php checked( $orientation, 'landscape' ); ?> value="landscape">
		</label>
	</p>

	<p>
		<input class="checkbox" type="checkbox" <?php checked( $motion, 'scroll' ); ?> 
			id="<?php echo esc_attr( $this->get_field_id( 'motion' ) ); ?>" 
			name="<?php echo esc_attr( $this->get_field_name( 'motion' ) ); ?>"
			value="scroll" /> 
		<label for="<?php echo esc_attr( $this->get_field_id( 'motion' ) ); ?>">Scroll?:</label>
	</p>

	<p>
		<label for="
		<?php
		echo esc_attr( $this->get_field_id( 'max-char' ) );
		?>
			">
			<?php echo 'Max chars:'; ?>			
			<input type="text" 
				id="<?php echo esc_attr( $this->get_field_id( 'max-char' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'max-char' ) ); ?>"
				value="<?php echo esc_attr( $max_char ); ?>" />			
		</label>
	</p> 

	<p>
		<label for="
		<?php
		echo esc_attr( $this->get_field_id( 'items-to-show' ) );
		?>
			">
			<?php echo 'Items to Show 1-5:'; ?>			
			<input type="text" 
				id="<?php echo esc_attr( $this->get_field_id( 'items-to-show' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'items-to-show' ) ); ?>"
				value="<?php echo esc_attr( $items_to_show ); ?>" />			
		</label>
	</p> 

	<p>
		<label for="
		<?php
		echo esc_attr( $this->get_field_id( 'scroll-speed' ) );
		?>
			">
			<?php echo 'Scroll delay 1000-6000 ms:'; ?>			
			<input type="text" 
				id="<?php echo esc_attr( $this->get_field_id( 'scroll-speed' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'scroll-speed' ) ); ?>"
				value="<?php echo esc_attr( $scroll_speed ); ?>" />			
		</label>
	</p> 

		<?php
		wp_reset_postdata();
	}

	/**
	 * Function to perform user input validation
	 *
	 * @param array $new_instance New settings.
	 * @param array $instance settings current settings.
	 * @return array Settings to use.
	 */
	public function update( $new_instance, $instance ) {

		$instance['widget_title'] = sanitize_text_field( $new_instance['widget_title'] );
		$instance['feedback_status'] = $new_instance['feedback_status'];
		$instance['orientation']     = $new_instance['orientation'];
		$instance['motion']          = $new_instance['motion'];

		if ( is_numeric( $new_instance['max-char'] ) && $new_instance['max-char'] > 0 ) {
			$instance['max-char'] = $new_instance['max-char'];
		} else {
			$instance['max-char'] = 150;
		}

		if ( is_numeric( $new_instance['items-to-show'] ) && $new_instance['items-to-show'] > 0 && $new_instance['items-to-show'] < 6 ) {
			$instance['items-to-show'] = $new_instance['items-to-show'];
		} else {
			$instance['items-to-show'] = 3;
		}

		if ( is_numeric( $new_instance['scroll-speed'] ) && $new_instance['scroll-speed'] >= 1000 && $new_instance['scroll-speed'] <= 6000 ) {
			$instance['scroll-speed'] = $new_instance['scroll-speed'];
		} else {
			$instance['scroll-speed'] = 3000;
		}

		return $instance;
	}

	/**
	 * Display contents of widget
	 *
	 * @param type $args Widget details.
	 * @param type $instance Settings.
	 */
	public function widget( $args, $instance ) {

		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';

		wp_enqueue_style( 'eq3f-feeback-recent-widget' );
		wp_enqueue_style( 'remsto-feedback-widget-item' );
		wp_enqueue_style( 'eq3f-base-style' );
		wp_enqueue_style( 'eq3f-item-style' );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_style( 'eq3f-rating-widget' );
		wp_enqueue_style( 'eq3f-icon-style' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'eq3f-more-popup-script' );
		wp_enqueue_script( 'eq3f-video-dialog' );
		wp_enqueue_script( 'eq3f-feedback-ajax-client-script' );

		wp_localize_script(
			'eq3f-feedback-ajax-client-script',
			'remsto_ajax_settings',
			array( 'ajax_url' => admin_url( 'admin-ajax.php' ) )
		);

		$orientation   = $instance['orientation'];
		$items_to_show = $instance['items-to-show'];
		$id            = $args['widget_id'];

		if ( 'scroll' === $instance['motion'] ) {
			wp_enqueue_script( 'eq3f-jquery-jcarousel' );
			wp_enqueue_script( 'eq3f-feedback-scroller' );
			$number_of_items = 10;
			$options         = array(
				'items_to_show'        => $items_to_show,
				'orientation'          => $orientation,
				'id'                   => $id,
				'jcarousel'            => array(
					'wrap'      => 'circular',
					'animation' => array(
						'duration' => 900,
						'easing'   => 'linear',
					),
				),
				'jcarousel_autoscroll' => array(
					'interval'  => $instance['scroll-speed'],
					'target'    => '+=1',
					'autostart' => true,
				),
			);

			if ( 'portrait' === $instance['orientation'] ) {
				$options['jcarousel']['vertical'] = true;
			} else {
				$options['jcarousel']['vertical'] = false;
			}

			wp_localize_script( 'eq3f-feedback-scroller', 'feedback_scroller_options', $options );
			$scroll_target = 'remsto-scroller';
		} else {
			$scroll_target   = 'remsto-static';
			$number_of_items = $instance['items-to-show'];
		}

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$testimonials     = $fc->getFeedback(
			$feedback_options['parent_type'],
			$feedback_options['parent_id'],
			$instance['feedback_status'],
			$number_of_items,
			0,
			null,
			0
		);
		$output           = '';

		if ( $testimonials ) {
			$output .= "<div class='remsto-feedback-$orientation-widget'>";

			if ( isset($instance['widget_title']) && '' !== $instance['widget_title'] ) {
				echo '<span class="latest-feedback-title">' . esc_attr( $instance['widget_title'] ) . '</span>';
			}

			if ( 'landscape' === $orientation ) {
				$output .= '<div class="feedback-carousel-ctl feedback-carousel-ctl-back" data-direction="back">';
				$output .= '</div>';
			}
			$output .= "<div class='feedback-$orientation-carousel-items feedback-carousel-items $scroll_target' id=$id data-items=$items_to_show >";
			$output .= "<ul class='feedback-$orientation-carousel'>";
			foreach ( $testimonials->data as $testimonial ) {
				$output .= '<li>';
				$output .= $this->render_feedback_item( $testimonial, $instance['max-char'] );
				$output .= '</li>';
			}
			$output .= '</ul>';
			if ( 'landscape' === $orientation ) {
				$output .= '</div>';
				$output .= '<div class="feedback-carousel-ctl feedback-carousel-ctl-forward" data-direction="forward">';
			}
			$output .= '</div>';
			$output .= '</div>';
		}
		echo $output;
	}

	/**
	 * Render feedback
	 *
	 * @param stdClass $testimonial Testimonial.
	 * @param int      $summary_description_char_limit Max characters in description.
	 * @return str Rendered feedback.
	 */
	private function render_feedback_item( $testimonial, $summary_description_char_limit ) {

		$created_date = gmdate( 'd/m/Y', $testimonial->created );
		if ( ! empty( $summary_description_char_limit ) ) {
			$stood_out = sanitize_text_field( RemstoModuleFunctions::truncate_utf8( $testimonial->stood_out, $summary_description_char_limit, true, true ) );
		} else {
			$stood_out = sanitize_text_field( $testimonial->stood_out );
		}

		if ( '' !== $testimonial->image_src ) {
			$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
			$image_src        = $feedback_options['remote_url'] . '/images/' . $testimonial->image_src;
		} else {
			$image_src = '';
		}

		$feedback_data = array(
			'title'      => sanitize_text_field( $testimonial->title ),
			'text'       => $stood_out,
			'anonymous'  => $testimonial->anonymous,
			'first_name' => sanitize_text_field( $testimonial->first_name ),
			'last_name'  => sanitize_text_field( $testimonial->last_name ),
			'town'       => sanitize_text_field( $testimonial->town ),
			'date'       => $created_date,
			'image_src'  => $image_src,
			'image_alt'  => $testimonial->image_alt ? sanitize_text_field( $testimonial->image_alt ) : '',
			'more_link'  => RemstoModuleFunctions::feedback_more_button(
				array(
					'id' => $testimonial->id,
				)
			),
		);

		if ( $testimonial->anonymous == 1 || $testimonial->concealed == 1 ) {
			    $feedback_data['first_name'] = 'Client details withheld';
		}
		else {
			if ( '' === $feedback_data['first_name'] ) {
				$feedback_data['first_name'] = 'Anonymous';
			} else {
				if ( '' === $feedback_data['town'] ) {
					$feedback_data['town'] = '&nbsp;';
				} else {
					$feedback_data['first_name'] = $feedback_data['first_name'] . '<span class="name-comma">,&nbsp;</span>';
				}
			}
		}

		if ( isset( $testimonial->website ) && strlen( $testimonial->website ) > 0 && '-' !== $testimonial->website ) {

			$http = preg_match( '/^http[^s]/i', $testimonial->website );
			if ( ! $http ) {
				$testimonial->website = 'http://' . $testimonial->website;
			}

			$link                          = "<a href='" . $testimonial->website . "' target='_blank' >Visit Website</a>";
			$feedback_data['website_link'] = RemstoModuleFunctions::feedback_button(
				array(
					'id'    => '',
					'label' => $link,
					'title' => 'visit website',
					'class' => 'remsto-weblink-btn',
				)
			);
		} else {
			$feedback_data['website_link'] = '&nbsp;';
		}

		if ( ! empty( $testimonial->video_testimonial ) ) {
			$feedback_data['video_testimonial'] =
			RemstoModuleFunctions::processVideoTestimonial( $testimonial->id, $testimonial->video_testimonial );
		} else {
			$feedback_data['video_testimonial'] = '&nbsp;';
		}

		return EQ3F_Feedback::compile_template(
			EQ3F_PLUGIN_PATH . 'widgets/views/remsto-feedback-block-item.tpl.php',
			$feedback_data );
	}

	/**
	 * Java Script for initialising carousel
	 *
	 * @param type $options Jcarousel options.
	 */
	private function init_carousel( $options ) {

		$options_jcarousel            = wp_json_encode( $options['jcarousel'] );
		$options_jcarousel_autoscroll = wp_json_encode( $options['jcarousel_autoscroll'] );
		$target                       = $options['target'];
		$options_string               = wp_json_encode( $options );
		$fun                          = "feedback_scroller_test($options_string)";
		echo '<script type="text/javascript"> jQuery(document).ready(function() {' . $fun . '});</script>';
	}

}
