<?php
/**
 * Performance widget
 */

/**
 * Performance widget
 */
class Eq3f_Widget_Performance extends WP_Widget {
 
  protected $field_names = array(
    'widget_title' => 'Widget Title',
    'excellent_label' => 'Excellent Label',
    'vgood_label' => 'Very Good Label',
    'average_label' => 'Average Label',
    'bad_label' => 'Bad Label',
    'terrible_label' => 'Terrible Label',
    'display_numbers' => 'Display Numbers',
  );
  protected $field_defaults = array(
    'widget_title' => '',
    'excellent_label' => 'Excellent',
    'vgood_label' => 'Very Good',
    'average_label' => 'Average',
    'bad_label' => 'Bad',
    'terrible_label' => 'Terrible',
    'display_numbers' => 'yes',
  );

	/**
	 * Constructor
	 */
	public function __construct() {

		parent::__construct(
			'eq3f_widget_performance',
			'Feedback Performance',
			array(
				'description' =>
								'Display Feedback performance',
			)
		);
	}

	/**
	 * Code to render options form
	 *
	 * @param arrray $instance Options.
	 */
	public function form( $instance ) {
  
    foreach($this->field_names as $field_index => $field_name) {
      $value = ( ! empty( $instance[$field_index] ) ?
      esc_attr( $instance[$field_index] ) :
      $this->field_defaults[$field_index] );

      print '<p>';
      print '<label for="'.esc_attr( $this->get_field_id( $field_index ) ).'">';
      print esc_attr( $field_name.':' );
			print '<input type="text" ';
		  print 'id="'.esc_attr( $this->get_field_id( $field_index ) ).'"';
			print 'name="'.esc_attr( $this->get_field_name( $field_index ) ).'"';
			print 'value="'.esc_attr( $value ).'" />';	
      print '</label>';
      print '</p>';
    }

		wp_reset_postdata();
	}


	/**
	 *
	 * Function to perform user input validation
	 *
	 * @param array $new_instance Options to test.
	 * @param array $instance Existing option.
	 * @return array Modified options.
	 */
	public function update( $new_instance, $instance ) {

    foreach($this->field_names as $field_index => $field_name) {
      $instance[$field_index] = sanitize_text_field( $new_instance[$field_index] );
    }

		return $instance;
	}

	/**
	 * Display contents of widget
	 *
	 * @param type $args Widget details.
	 * @param type $instance Settings.
	 */
	public function widget( $args, $instance ) {

		wp_enqueue_style( 'eq3f-performance-widget' );

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$performance_data = $fc->getPerformance( $feedback_options['parent_type'], $feedback_options['parent_id'] );

		print '<div class="performance-widget">';

		if ( '' !== $instance['widget_title'] ) {
			print '<span class="feedback-performance-title">' . esc_attr( $instance['widget_title'] ) . '</span>';
		}

		$total = $performance_data->total;
		unset( $performance_data->total );

		foreach ( $performance_data as $name => $performance ) {
			// convert to percentage.
			if ( $total > 0 ) {
				$percentage = round( ( ( $performance / $total ) * 100 ), 0 );
			} else {
				$percentage = 0;
			}

			$bar_data = array(
				'label'       => (isset($instance[ $name.'_label' ]))?esc_attr( $instance[ $name.'_label' ]):$this->field_defaults[ $name.'_label' ],
				'name'        => $name,
				'percentage'  => $percentage,
				'performance' => $performance,
				'display_numbers' => (isset($instance[ 'display_numbers' ]))?esc_attr( $instance[ 'display_numbers' ]):$this->field_defaults[ 'display_numbers' ],
			);

			EQ3F_Feedback::render_template(
				EQ3F_PLUGIN_PATH . 'widgets/views/remsto-client-feedback-performance-bar.tpl.php',
				$bar_data );
		}

		print '</div>';
	}

}
