<?php
/**
 * Performance widget
 */

/**
 * Performance widget
 */
class Eq3f_Widget_Rating extends WP_Widget {
 
  protected $field_names = array(
    'widget_title' => 'Widget Title',
    'promised_label' => 'Promised Label',
    'value_label' => 'Value Label',
    'helpful_label' => 'Helpful Label',
    'communication_label' => 'Communication Label',
    'total_label' => 'Total Label',
    'display_numbers' => 'Display Numbers',
  );
  protected $field_defaults = array(
    'widget_title' => '',
    'promised_label' => 'Delivered as promised',
    'value_label' => 'Percieved value',
    'helpful_label' => 'Considerate and helpful',
    'communication_label' => 'Deliver a high standard',
		'total_label' => 'Total',
    'display_numbers' => 'yes',
  );

	/**
	 * Constructor
	 */
	public function __construct() {

		parent::__construct(
			'eq3f_widget_rating',
			'Feedback Rating',
			array(
				'description' =>
								'Display Feedback Rating',
			)
		);
	}

	/**
	 * Code to render options form
	 *
	 * @param arrray $instance Options.
	 */
	public function form( $instance ) {
  
    foreach($this->field_names as $field_index => $field_name) {
      $value = ( ! empty( $instance[$field_index] ) ?
      esc_attr( $instance[$field_index] ) :
      $this->field_defaults[$field_index] );

      print '<p>';
      print '<label for="'.esc_attr( $this->get_field_id( $field_index ) ).'">';
      print esc_attr( $field_name.':' );
			print '<input type="text" ';
		  print 'id="'.esc_attr( $this->get_field_id( $field_index ) ).'"';
			print 'name="'.esc_attr( $this->get_field_name( $field_index ) ).'"';
			print 'value="'.esc_attr( $value ).'" />';	
      print '</label>';
      print '</p>';
    }

		wp_reset_postdata();
	}

	/**
	 *
	 * Function to perform user input validation
	 *
	 * @param array $new_instance Options to test.
	 * @param array $instance Existing option.
	 * @return array Modified options.
	 */
	public function update( $new_instance, $instance ) {

    foreach($this->field_names as $field_index => $field_name) {
      $instance[$field_index] = sanitize_text_field( $new_instance[$field_index] );
    }

		return $instance;
	}

	/**
	 * Display contents of widget
	 *
	 * @param type $args Widget details.
	 * @param type $instance Settings.
	 */
	public function widget( $args, $instance ) {

		wp_enqueue_style( 'eq3f-rating-widget' );
		require_once EQ3F_PLUGIN_PATH . 'vendor/enquir3/RemstoModuleFunctions.inc';

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$rating_data      = $fc->getRatings( $feedback_options['parent_type'], $feedback_options['parent_id'] );

		$output = '<div class="rating-widget">';

		if ( '' !== $instance['widget_title'] ) {
			$output .= '<div class="feedback-rating-title">' . $instance['widget_title'] . '</div>';
		}

		// @todo get this as on option.
		$only_total = false;

		if ( $only_total ) {
			unset( $rating_data->promised );
			unset( $rating_data->value );
			unset( $rating_data->helpful );
			unset( $rating_data->communication );
		}

		foreach ( $rating_data as $key => $rating ) {
			$rating_percent = round( $rating, 0 );

			$variables = array(
				'rating' => $rating_percent,
				'key'    => $key,
				'label'       => (isset($instance[ $key.'_label' ]))?esc_attr( $instance[ $key.'_label' ]):$this->field_defaults[ $key.'_label' ],
				'display_numbers' => (isset($instance[ 'display_numbers' ]))?esc_attr( $instance[ 'display_numbers' ]):$this->field_defaults[ 'display_numbers' ],
				'stars'  => 5,
			);

			$output .= RemstoModuleFunctions::feedback_rating_display( $variables );
		}

		$output .= '</div>';
		echo $output;
	}

}
