<?php
/**
 * Summary widget
 */

/**
 * Summary widget
 */
class Eq3f_Widget_Summary extends WP_Widget {
 
  protected $field_names = array(
    'widget_title' => 'Widget Title',
    'before_label' => 'Before Label',
    'after_label' => 'After Label',
  );
  protected $field_defaults = array(
    'widget_title' => '',
    'before_label' => 'Feedback received from %n people.',
    'after_label' => 'would recommend this business.',
  );

	/**
	 * Constructor
	 */
	public function __construct() {

		parent::__construct(
			'eq3f_widget_summary',
			'Feedback Summary',
			array(
				'description' =>
								'Display a summary of all Feedback',
			)
		);
	}


	/**
	 * Code to render options form
	 *
	 * @param arrray $instance Options.
	 */
	public function form( $instance ) {
  
    foreach($this->field_names as $field_index => $field_name) {
      $value = ( ! empty( $instance[$field_index] ) ?
      esc_attr( $instance[$field_index] ) :
      $this->field_defaults[$field_index] );

      print '<p>';
      print '<label for="'.esc_attr( $this->get_field_id( $field_index ) ).'">';
      print esc_attr( $field_name.':' );
			print '<input type="text" ';
		  print 'id="'.esc_attr( $this->get_field_id( $field_index ) ).'"';
			print 'name="'.esc_attr( $this->get_field_name( $field_index ) ).'"';
			print 'value="'.esc_attr( $value ).'" />';	
      print '</label>';
      print '</p>';
    }

		wp_reset_postdata();
	}

	/**
	 *
	 * Function to perform user input validation
	 *
	 * @param array $new_instance Options to test.
	 * @param array $instance Existing option.
	 * @return array Modified options.
	 */
	public function update( $new_instance, $instance ) {

    foreach($this->field_names as $field_index => $field_name) {
      $instance[$field_index] = sanitize_text_field( $new_instance[$field_index] );
    }

		return $instance;
	}

	/**
	 * Display contents of widget
	 *
	 * @param type $args Widget details.
	 * @param type $instance Settings.
	 */
	public function widget( $args, $instance ) {

		wp_enqueue_style( 'eq3f-feedback' );

		$feedback_options = get_option( 'enquir3_feedback_options_new', array() );
		$fc               = new RemstoFeedbackClient( $feedback_options );
		$data             = $fc->getFeedbackSummary( $feedback_options['parent_type'], $feedback_options['parent_id'] );

		$total     = 0;
		$recommend = 0;

		foreach ( $data as $row ) {
			$total += $row->total;
			if ( '1' === $row->recommend ) {
				$recommend = $row->total;
			}
		}

		$summary_data = array(
			'total_feedback_count'  => $total,
			'would_recommend_count' => $recommend,
			'title'                 => $instance['widget_title'],
			'before_label'          => str_replace( '%n', $total, (isset($instance['before_label']))?$instance['before_label']:$this->field_defaults['before_label'] ),
			'after_label'           => str_replace( '%n', $total, (isset($instance['after_label']))?$instance['after_label']:$this->field_defaults['after_label'] ),
		);

		EQ3F_Feedback::render_template(
			EQ3F_PLUGIN_PATH . 'widgets/views/feedback-widget-summary.tpl.php',
			$summary_data );
	}

}
